#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Get Bookkeeping paths given a decay.

@author Vanya BELYAEV Ivan.Belyaev@itep.ru
        Federico Stagni fstagni@cern.ch
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import six
import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from DIRAC.Core.Base import Script
  from LHCbDIRAC.BookkeepingSystem.Client.BookkeepingClient import BookkeepingClient

  Script.setUsageMessage(__doc__ + '\n'.join([
      'Usage:',
      '  %s eventType  ' % Script.scriptName]))

  Script.registerSwitch('p', 'production', "Obtain the paths in ``Production format'' for Ganga")
  Script.parseCommandLine(ignoreErrors=True)

  productionFormat = False
  for p, _v in Script.getUnprocessedSwitches():
    if p.lower() in ('p', 'production'):
      productionFormat = True
    break

  args = Script.getPositionalArgs()
  if len(args) < 1:
    Script.showHelp(exitCode=1)

  eventType = args[0]

  bkClient = BookkeepingClient()

  # # get productions for given event type
  res = bkClient.getProductionSummaryFromView({'EventType': eventType, 'Visible': True})
  if not res['OK']:
    gLogger.error('Could not retrieve production summary for event %s' % eventType, res['Message'])
    DIRAC.exit(1)
  prods = res['Value']

  # # get production-IDs
  prodIDs = [p['Production'] for p in prods]

  # # loop over all productions
  for prodID in sorted(prodIDs):

    res = bkClient.getProductionInformation(prodID)
    if not res['OK']:
      gLogger.error('Could not retrieve production infos for production %s' % prodID, res['Message'])
      continue
    prodInfo = res['Value']
    steps = prodInfo['Steps']
    if isinstance(steps, six.string_types):
      continue
    files = prodInfo["Number of files"]
    events = prodInfo["Number of events"]
    path = prodInfo["Path"]
    dddb = prodInfo["Steps"][0][4]
    conddb = prodInfo["Steps"][0][5]

    evts = 0
    ftype = None
    for i in events:
      if i[0] in ['GAUSSHIST', 'LOG', 'SIM', 'DIGI']:
        continue
      evts += i[1]
      if not ftype:
        ftype = i[0]

    nfiles = 0
    for f in files:
      if f[1] in ['GAUSSHIST', 'LOG', 'SIM', 'DIGI']:
        continue
      if f[1] != ftype:
        continue
      nfiles += f[0]

    p0, n, p1 = path.partition('\n')
    if n:
      path = p1

    if productionFormat:
      p, s, e = path.rpartition('/')
      if s and e:
        path = '/%d/%d/%s' % (prodID, int(eventType), e)
    print(path, dddb, conddb, nfiles, evts, prodID)


if __name__ == "__main__":
  main()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Generic XMLParser used to convert a XML file into more pythonic style.

A tree ox XMLNodes.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import xml.dom.minidom

__RCSID__ = "$Id$"


################################################################################

class XMLNode(object):
  """XMLNodes represent XML elements.

  May have attributes. They have either children or value ( exclusive-
  or).
  """

  def __init__(self, name):
    self.name = name
    self.attributes = {}
    self.children = None
    self.value = None

  def childrens(self, name):
    """return children."""
    return [child for child in self.children if child.name == name]

  def __repr__(self):
    return '< %s >' % self.name

################################################################################


class XMLTreeParser(object):
  """XMLTreeParser converts an XML file or a string into a tree of XMLNodes. It
  does not validate the XML.

  Elements that are the only child of an element and are of text or
  cdata type are considered to be the value of their parent.
  """

  def __init__(self):
    self.tree = None

  def parse(self, xmlFile):
    """parse the XML."""
    domXML = xml.dom.minidom.parse(xmlFile)
    self.__handleXML(domXML)
    return self.tree

  def parseString(self, xmlString):
    """parse the XML."""
    domXML = xml.dom.minidom.parseString(xmlString)
    self.__handleXML(domXML)
    return self.tree

################################################################################
# AUXILIAR FUNCTIONS
################################################################################

  def __handleXML(self, domXML):
    """handles first child."""
    self.tree = self.__handleElement([domXML.firstChild])

  def __handleElement(self, elements):
    """treat each element."""
    nodes = []

    for el in elements:

      if el.nodeType == el.TEXT_NODE:
        continue

      node = XMLNode(el.localName)
      node.attributes = self.__getAttributesDict(el)

      if len(el.childNodes) == 1:
        childNode = el.childNodes[0]
        if childNode.nodeType == childNode.TEXT_NODE or childNode.nodeType == childNode.CDATA_SECTION_NODE:
          node.value = self.__handleTextElement(el.childNodes[0])
        else:
          node.children = self.__handleElement(el.childNodes)
      else:
        node.children = self.__handleElement(el.childNodes)

      nodes.append(node)

    return nodes

  @staticmethod
  def __getAttributesDict(element):
    """get the attributes in a dictionary."""
    dictionary = {}
    if element.attributes:
      for attr in element.attributes.values():
        dictionary[attr.name] = attr.value
    return dictionary

  def __handleTextElement(self, textElement):
    """treat the Text element."""
    return self.__getText(textElement)

  @staticmethod
  def __getText(node):
    """get the TEXT."""
    data = ''
    if node.nodeType == node.TEXT_NODE or node.nodeType == node.CDATA_SECTION_NODE:
      data = node.data
    return data

# Utilies for XML Report


def addChildNode(parentNode, tag, returnChildren, args):
  """
  :param parentNode:
    node where the new node is going to be appended
  :param tag:
    name if the XML element to be created
  :param returnChildren:
    flag to return or not the children node, used to avoid unused variables
  :param args:
    possible attributes of the element
  """

  allowedTags = ['Job', 'TypedParameter', 'InputFile', 'OutputFile', 'Parameter', 'Replica', 'SimulationCondition']

  def genJobDict(configName, configVersion, ldate, ltime):
    return {"ConfigName": configName,
            "ConfigVersion": configVersion,
            "Date": ldate,
            "Time": ltime}

  def genTypedParameterDict(name, value, typeP="Info"):
    return {"Name": name,
            "Value": value,
            "Type": typeP}

  def genInputFileDict(name):
    return {"Name": name}

  def genOutputFileDict(name, typeName, typeVersion):
    return {"Name": name,
            "TypeName": typeName,
            "TypeVersion": typeVersion}

  def genParameterDict(name, value):
    return {"Name": name,
            "Value": value}

  def genReplicaDict(name, location="Web"):
    return {"Name": name,
            "Location": location}

  def genSimulationConditionDict():
    return {}

  if tag not in allowedTags:
    # We can also return S_ERROR, but this let's the job keep running.
    tagsDict = {}
  else:
    tagsDict = locals()['gen%sDict' % tag](*args)

  childNode = xml.dom.minidom.Document().createElement(tag)
  for key, value in tagsDict.items():
    childNode.setAttribute(key, str(value))
  parentNode.appendChild(childNode)

  if returnChildren:
    return (parentNode, childNode)
  return parentNode



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""StepAccounting module performs several common operations at the end of a
workflow step, in particular prepares and sends the step accounting data."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC import S_OK, S_ERROR, gConfig, gLogger
from DIRAC.Core.Utilities import Time
from DIRAC.Workflow.Utilities.Utils import getStepCPUTimes

from LHCbDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from LHCbDIRAC.AccountingSystem.Client.Types.JobStep import JobStep


class StepAccounting(ModuleBase):
  """StepAccounting class."""

  def __init__(self, bkClient=None, dm=None):

    self.log = gLogger.getSubLogger("StepAccounting")
    super(StepAccounting, self).__init__(self.log, bkClientIn=bkClient, dm=dm)

    self.dsc = None
    self.stepStat = None

    self.version = __RCSID__

  ########################################################################

  def _resolveInputVariables(self, dsc=None):
    """By convention all workflow parameters are resolved here."""

    super(StepAccounting, self)._resolveInputVariables()
    super(StepAccounting, self)._resolveInputStep()

    if dsc is not None:
      self.dsc = dsc
    else:
      self.dsc = self.workflow_commons['AccountingReport']

    if self.stepStatus['OK']:
      self.stepStat = 'Done'
    else:
      self.stepStat = 'Failed'

  ########################################################################

  def execute(self, production_id=None, prod_job_id=None, wms_job_id=None,
              workflowStatus=None, stepStatus=None,
              wf_commons=None, step_commons=None,
              step_number=None, step_id=None,
              js=None, xf_o=None, dsc=None):

    try:
      super(StepAccounting, self).execute(self.version, production_id, prod_job_id, wms_job_id,
                                          workflowStatus, stepStatus,
                                          wf_commons, step_commons, step_number, step_id)

      # Check if the step is worth accounting
      if 'applicationName' not in self.step_commons:
        self.log.info('Not an application step: it will not be accounted')
        return S_OK()

      ########################################################################
      # Timing
      execTime, cpuTime = getStepCPUTimes(self.step_commons)
      normCPU = cpuTime
      cpuNormFactor = gConfig.getValue("/LocalSite/CPUNormalizationFactor", 0.0)
      if cpuNormFactor:
        normCPU = cpuTime * cpuNormFactor

      if not js:
        jobStep = JobStep()
      else:
        jobStep = js

      if not xf_o:
        try:
          xf_o = self.step_commons['XMLSummary_o']
        except KeyError:
          self.log.error('XML Summary object could not be found (not produced?), skipping the report')
          return S_OK()

      self._resolveInputVariables(dsc)

      now = Time.dateTime()
      jobStep.setStartTime(now)
      jobStep.setEndTime(now)

      dataDict = {'JobGroup': str(self.production_id),
                  'RunNumber': self.runNumber,
                  'EventType': self.eventType,
                  'ProcessingType': self.stepProcPass,  # this is the processing pass of the step
                  'ProcessingStep': self.BKstepID,  # the step ID
                  'Site': self.siteName,
                  'FinalStepState': self.stepStat,

                  'CPUTime': cpuTime,
                  'NormCPUTime': normCPU,
                  'ExecTime': execTime * self.numberOfProcessors,
                  'InputData': sum(xf_o.inputFileStats.values()),
                  'OutputData': sum(xf_o.outputFileStats.values()),
                  'InputEvents': xf_o.inputEventsTotal,
                  'OutputEvents': xf_o.outputEventsTotal}

      jobStep.setValuesFromDict(dataDict)

      res = jobStep.checkValues()
      if not res['OK']:
        self.log.error('Values for StepAccounting are wrong', res['Message'], dataDict)
        return S_ERROR('Values for StepAccounting are wrong')

      if not self._enableModule():
        self.log.info('Not enabled, would have accounted for %s' % dataDict)
        return S_OK()

      self.dsc.addRegister(jobStep)

      return S_OK()

    except Exception as e:  # pylint:disable=broad-except
      self.log.exception("Failure in StepAccounting execute module", lException=e)
      return S_ERROR(str(e))

    finally:
      super(StepAccounting, self).finalize(self.version)



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Returns the platform supported by the current WN."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import LbPlatformUtils
from DIRAC.Core.Utilities.DIRACScript import DIRACScript


def sendMail(msg=''):
  """send a notification mail when no platform is found."""
  from DIRAC.FrameworkSystem.Client.NotificationClient import NotificationClient
  from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
  from DIRAC import gConfig

  mailAddress = Operations().getValue('EMail/JobFailures', 'Vladimir.Romanovskiy@cern.ch')
  site = gConfig.getValue('LocalSite/Site')
  ce = gConfig.getValue('LocalSite/GridCE')
  queue = gConfig.getValue('LocalSite/CEQueue')
  body = "*** THIS IS AN AUTOMATED MESSAGE ***" + '\n\n' + msg + '\n\n'
  body = body + "At site %s, CE = %s, queue = %s" % (site, ce, queue) + '\n\n'

  for mA in mailAddress.replace(' ', '').split(','):
    NotificationClient().sendMail(mailAddress, "Problem with DIRAC architecture",
                                  body, 'federico.stagni@cern.ch', localAttempt=False)


@DIRACScript()
def main():
  from DIRAC.Core.Base import Script
  Script.registerSwitch('', 'BinaryTag', '   Print the host binary tag instead of the host dirac_platform')
  Script.parseCommandLine(ignoreErrors=True)

  from DIRAC import gConfig, gLogger, exit as dExit
  from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations

  parList = Script.getUnprocessedSwitches()
  for switch, _val in parList:
    if switch == 'BinaryTag':
      try:
        # Get the binaryTag name. If an error occurs, an exception is thrown
        binaryTag = LbPlatformUtils.host_binary_tag()
        if not binaryTag:
          gLogger.fatal("There is no binaryTag corresponding to this machine")
          sendMail("There is no binaryTag corresponding to this machine")
          dExit(1)
        print(binaryTag)
        dExit(0)
      except Exception as e:
        msg = "Exception getting binaryTag: " + repr(e)
        gLogger.exception(msg, lException=e)
        sendMail(msg)
        dExit(1)

  try:
    site = gConfig.getValue('/LocalSite/Site')
    grid = site.split('.')[0]
    ce = gConfig.getValue('/LocalSite/GridCE')

    allowContainers = gConfig.getValue('/Resources/Sites/%s/%s/CEs/%s/AllowContainers' % (grid, site, ce), None)
    if allowContainers is None:
      allowContainers = gConfig.getValue('/Resources/Sites/%s/%s/AllowContainers' % (grid, site), None)
    if allowContainers is None:
      allowContainers = Operations().getValue('GaudiExecution/AllowContainers', 'no')

    if allowContainers.lower() in ('yes', 'true', 'all'):
      allowContainers = True
    elif allowContainers.lower() in ('no', 'false', 'none', ''):
      allowContainers = False
    else:
      gLogger.warn("Invalid value for AllowContainers", repr(allowContainers))
      allowContainers = False

    # Get the platform name. If an error occurs, an exception is thrown
    platform = LbPlatformUtils.dirac_platform(allow_containers=allowContainers)
    if not platform:
      gLogger.fatal("There is no platform corresponding to this machine")
      sendMail("There is no platform corresponding to this machine")
      dExit(1)
    print(platform)
    dExit(0)

  except Exception as e:
    msg = "Exception getting platform: " + repr(e)
    gLogger.exception(msg, lException=e)
    sendMail(msg)
    dExit(1)


if __name__ == "__main__":
  main()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Unittest for: LHCbDIRAC.AccountingSystem.private.Plotters.JobStepPlotter.

JobStepPlotter.__bases__:
  DIRAC.AccountingSystem.private.Plotters.BaseReporter

We are assuming there is a solid test of __bases__, we are not testing them
here and assuming they work fine.

IMPORTANT: the test MUST be pylint compliant !
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import mock
import unittest


class JobStepPlotterTestCase(unittest.TestCase):
  """JobStepPlotterTestCase."""

  moduleTested = None
  classsTested = None

  def mockModuleTested(self, moduleTested):
    """Used to not redo the mocking done on the parent class ( if any )"""

    # Tries to get the mocks of the parent TestCases ( if any )
    for baseClass in self.__class__.__bases__:
      try:
        # pylint: disable=no-member
        moduleTested = baseClass.mockModuleTested(moduleTested)
      except TypeError:
        continue

    # And then makes its own mock
    class MockJobStep:
      # pylint: disable=missing-docstring,no-init
      definitionKeyFields = ('JobGroup', 'RunNumber', 'EventType', 'ProcessingType',
                             'ProcessingStep', 'Site', 'FinalStepState')

    moduleTested.JobStep = mock.Mock(return_value=MockJobStep())

    return moduleTested

  def setUp(self):
    """Setup the test case."""

    import LHCbDIRAC.AccountingSystem.private.Plotters.JobStepPlotter as moduleTested

    self.moduleTested = self.mockModuleTested(moduleTested)
    self.classsTested = self.moduleTested.JobStepPlotter

  def tearDown(self):
    """Tear down the test case."""

    del self.moduleTested
    del self.classsTested

# ...............................................................................


class JobStepPlotterUnitTest(JobStepPlotterTestCase):
  """JobStepPlotterUnitTest.

  <constructor>
   - test_instantiate
  <class variables>
   - test_typeName
   - test_typeKeyFields
   - test_reportCPUEfficiencyName
   - test_reportCPUUsageName
   - test_reportPieCPUTimeName
   - test_reportCumulativeCPUTimeName
   - test_reportNormCPUTimeName
   - test_reportCumulativeNormCPUTimeName
   - test_reportPieNormCPUTimeName
   - test_reportInputDataName
   - test_reportCumulativeInputDataName
   - test_reportPieInputDataName
   - test_reportOutputDataName
   - test_reportCumulativeOutputDataName
   - test_reportPieOutputDataName
   - test_reportInputEventsName
   - test_reportCumulativeInputEventsName
   - test_reportPieInputEventsName
   - test_reportOutputEventsName
   - test_reportCumulativeOutputEventsName
   - test_reportPieOutputEventsName
   - test_reportInputEventsPerOutputEventsName
   - test_reportCPUTimePerOutputEventsName
   - test_reportCPUTimePerInputEventsName
  <methods>
  """

  def test_instantiate(self):
    """tests that we can instantiate one object of the tested class."""
    obj = self.classsTested(None, None)
    self.assertEqual('JobStepPlotter', obj.__class__.__name__,
                     msg='Expected JobStepPlotter object')

  def test_typeName(self):
    """test the class variable "_typeName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._typeName, "JobStep",
                     msg='Expected JobStep as value')

  def test_typeKeyFields(self):
    """test the class variable "_typeKeyFields"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._typeKeyFields, ['JobGroup', 'RunNumber', 'EventType',
                                          'ProcessingType', 'ProcessingStep',
                                          'Site', 'FinalStepState'],
                     msg='Expected keys from MockJobStep')

  def test_reportCPUEfficiencyName(self):
    """test the class variable "_reportCPUEfficiencyName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCPUEfficiencyName, "CPU efficiency",
                     msg='Expected CPU efficiency as value')

  def test_reportCPUUsageName(self):
    """test the class variable "_reportCPUUsageName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCPUUsageName, "CPU time",
                     msg='Expected CPU time as value')

  def test_reportPieCPUTimeName(self):
    """test the class variable "_reportPieCPUTimeName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportPieCPUTimeName, "Pie plot of CPU time",
                     msg='Expected Pie plot of CPU time as value')

  def test_reportCumulativeCPUTimeName(self):
    """test the class variable "_reportCumulativeCPUTimeName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCumulativeCPUTimeName, "Cumulative CPU time",
                     msg='Expected Cumulative CPU time as value')

  def test_reportNormCPUTimeName(self):
    """test the class variable "_reportNormCPUTimeName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportNormCPUTimeName, "NormCPU time",
                     msg='Expected NormCPU time as value')

  def test_reportCumulativeNormCPUTimeName(self):
    """test the class variable "_reportCumulativeNormCPUTimeName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCumulativeNormCPUTimeName, "Cumulative normalized CPU time",
                     msg='Expected Cumulative normalized CPU time as value')

  def test_reportPieNormCPUTimeName(self):
    """test the class variable "_reportPieNormCPUTimeName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportPieNormCPUTimeName, "Pie plot of NormCPU time",
                     msg='Expected Pie plot of NormCPU time as value')

  def test_reportInputDataName(self):
    """test the class variable "_reportInputDataName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportInputDataName, "Input Data",
                     msg='Expected Input Data as value')

  def test_reportCumulativeInputDataName(self):
    """test the class variable "_reportCumulativeInputDataName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCumulativeInputDataName, "Cumulative Input Data",
                     msg='Expected Cumulative Input Data as value')

  def test_reportPieInputDataName(self):
    """test the class variable "_reportPieInputDataName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportPieInputDataName, "Pie plot of Input Data",
                     msg='Expected Pie plot of Input Data as value')

  def test_reportOutputDataName(self):
    """test the class variable "_reportOutputDataName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportOutputDataName, "Output Data",
                     msg='Expected Output Data as value')

  def test_reportCumulativeOutputDataName(self):
    """test the class variable "_reportCumulativeOutputDataName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCumulativeOutputDataName, "Cumulative OutputData",
                     msg='Expected Cumulative OutputData as value')

  def test_reportPieOutputDataName(self):
    """test the class variable "_reportPieOutputDataName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportPieOutputDataName, "Pie plot of Output Data",
                     msg='Expected Pie plot of Output Data as value')

  def test_reportInputEventsName(self):
    """test the class variable "_reportInputEventsName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportInputEventsName, "Input Events",
                     msg='Expected Input Events as value')

  def test_reportCumulativeInputEventsName(self):
    """test the class variable "_reportCumulativeInputEventsName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCumulativeInputEventsName, "Cumulative Input Events",
                     msg='Expected Cumulative Input Events as value')

  def test_reportPieInputEventsName(self):
    """test the class variable "_reportPieInputEventsName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportPieInputEventsName, "Pie plot of Input Events",
                     msg='Expected Pie plot of Input Events as value')

  def test_reportOutputEventsName(self):
    """test the class variable "_reportOutputEventsName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportOutputEventsName, "Output Events",
                     msg='Expected Output Events as value')

  def test_reportCumulativeOutputEventsName(self):
    """test the class variable "_reportCumulativeOutputEventsName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCumulativeOutputEventsName, "Cumulative Output Events",
                     msg='Expected Cumulative Output Events as value')

  def test_reportPieOutputEventsName(self):
    """test the class variable "_reportPieOutputEventsName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportPieOutputEventsName, "Pie plot of Output Events",
                     msg='Expected Pie plot of Output Events as value')

  def test_reportInputEventsPerOutputEventsName(self):
    """test the class variable "_reportInputEventsPerOutputEventsName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportInputEventsPerOutputEventsName, "Input/Output Events",
                     msg='Expected Input/Output Events as value')

  def test_reportCPUTimePerOutputEventsName(self):
    """test the class variable "_reportCPUTimePerOutputEventsName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCPUTimePerOutputEventsName, "CPUTime/Output Events",
                     msg='Expected CPUTime/Output Events as value')

  def test_reportCPUTimePerInputEventsName(self):
    """test the class variable "_reportCPUTimePerInputEventsName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCPUTimePerInputEventsName, "CPUTime/Input Events",
                     msg='Expected CPUTime/Input Events as value')

  # FIXME: add crashes !

################################################################################


if __name__ == '__main__':
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(JobStepPlotterTestCase)
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(JobStepPlotterUnitTest))
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Client test."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# pylint: disable=protected-access, missing-docstring, invalid-name, line-too-long

import unittest

from mock import MagicMock

from DIRAC import S_OK, gLogger

from LHCbDIRAC.TransformationSystem.Client.TaskManager import LHCbWorkflowTasks
from LHCbDIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
from LHCbDIRAC.TransformationSystem.Utilities.PluginUtilities import PluginUtilities, groupByRun


def getSitesForSE(ses):
  if ses == 'pippo':
    return {'OK': True, 'Value': ['Site2', 'Site3']}
  else:
    return {'OK': True, 'Value': ['Site3']}


class ClientTestCase(unittest.TestCase):

  def setUp(self):
    tcMock = MagicMock()
    sc = MagicMock()
    jmc = MagicMock()

    self.l_wft = LHCbWorkflowTasks(tcMock, submissionClient=sc, jobMonitoringClient=jmc)
    self.tc = TransformationClient()
    self.tc.dataProcessingTypes = ['MCSimulation', 'DataReconstruction']

    self.tsMock = MagicMock()

    self.fcMock = MagicMock()
    self.fcMock.getFileSize.return_value = S_OK({'Failed': [], 'Successful': cachedLFNSize})

    gLogger.setLevel('DEBUG')

    self.maxDiff = None

  def tearDown(self):
    pass


class TaskManagerSuccess(ClientTestCase):
  pass


class TransformationClientSuccess(ClientTestCase):

  def test__applyTransformationFilesStateMachine(self):
    tsFiles = {}
    dictOfNewLFNsStatus = {}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {})

    tsFiles = {}
    dictOfNewLFNsStatus = {'foo': ['status', 2, 1234]}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {})

    tsFiles = {'foo': ['status', 2, 1234]}
    dictOfNewLFNsStatus = {'foo': 'status'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {})

    tsFiles = {'foo': ['status', 2, 1234]}
    dictOfNewLFNsStatus = {'foo': 'statusA'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {'foo': 'statusA'})

    tsFiles = {'foo': ['status', 2, 1234], 'bar': ['status', 2, 5678]}
    dictOfNewLFNsStatus = {'foo': 'status'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {})

    tsFiles = {'foo': ['status', 2, 1234], 'bar': ['status', 2, 5678]}
    dictOfNewLFNsStatus = {'foo': 'statusA'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {'foo': 'statusA'})

    tsFiles = {'foo': ['status', 2, 1234], 'bar': ['status', 2, 5678]}
    dictOfNewLFNsStatus = {'foo': 'A', 'bar': 'B'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {'foo': 'A', 'bar': 'B'})

    tsFiles = {'foo': ['status', 2, 1234]}
    dictOfNewLFNsStatus = {'foo': 'A', 'bar': 'B'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {'foo': 'A'})

    tsFiles = {'foo': ['Assigned', 2, 1234]}
    dictOfNewLFNsStatus = {'foo': 'A', 'bar': 'B'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {'foo': 'A'})

    tsFiles = {'foo': ['Assigned', 2, 1234], 'bar': ['Assigned', 2, 5678]}
    dictOfNewLFNsStatus = {'foo': 'Assigned', 'bar': 'Processed'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {'bar': 'Processed'})

    tsFiles = {'foo': ['Processed', 2, 1234], 'bar': ['Unused', 2, 5678]}
    dictOfNewLFNsStatus = {'foo': 'Assigned', 'bar': 'Processed'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {'bar': 'Processed'})

    tsFiles = {'foo': ['Processed', 2, 1234], 'bar': ['Unused', 2, 5678]}
    dictOfNewLFNsStatus = {'foo': 'Assigned', 'bar': 'Processed'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, True)
    self.assertEqual(res, {'foo': 'Assigned', 'bar': 'Processed'})

    tsFiles = {'foo': ['Processed', 2, 1234], 'bar': ['Unused', 2, 5678]}
    dictOfNewLFNsStatus = {'foo': 'Assigned', 'bar': 'Processed'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, True)
    self.assertEqual(res, {'foo': 'Assigned', 'bar': 'Processed'})

    tsFiles = {'foo': ['MaxReset', 12, 1234], 'bar': ['Processed', 22, 5678]}
    dictOfNewLFNsStatus = {'foo': 'Unused', 'bar': 'Unused'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {})

    tsFiles = {'foo': ['MaxReset', 12, 1234], 'bar': ['Processed', 22, 5678]}
    dictOfNewLFNsStatus = {'foo': 'Unused', 'bar': 'Unused'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, True)
    self.assertEqual(res, {'foo': 'Unused', 'bar': 'Unused'})

    tsFiles = {'foo': ['Assigned', 20, 1234], 'bar': ['Processed', 2, 5678]}
    dictOfNewLFNsStatus = {'foo': 'Unused', 'bar': 'Unused'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, False)
    self.assertEqual(res, {'foo': 'MaxReset'})

    tsFiles = {'foo': ['Assigned', 20, 1234], 'bar': ['Processed', 2, 5678]}
    dictOfNewLFNsStatus = {'foo': 'Unused', 'bar': 'Unused'}
    res = self.tc._applyTransformationFilesStateMachine(tsFiles, dictOfNewLFNsStatus, True)
    self.assertEqual(res, {'foo': 'Unused', 'bar': 'Unused'})

  def test__applyTransformationStatusStateMachine(self):
    transIDAsDict = {123: ['Active', 'MCSimulation']}
    dictOfProposedstatus = {123: 'Stopped'}
    res = self.tc._applyTransformationStatusStateMachine(transIDAsDict, dictOfProposedstatus, False)
    self.assertEqual(res, 'Stopped')

    transIDAsDict = {123: ['New', 'MCSimulation']}
    dictOfProposedstatus = {123: 'Active'}
    res = self.tc._applyTransformationStatusStateMachine(transIDAsDict, dictOfProposedstatus, False)
    self.assertEqual(res, 'Active')

    transIDAsDict = {123: ['New', 'MCSimulation']}
    dictOfProposedstatus = {123: 'New'}
    res = self.tc._applyTransformationStatusStateMachine(transIDAsDict, dictOfProposedstatus, False)
    self.assertEqual(res, 'New')

    transIDAsDict = {123: ['New', 'MCSimulation']}
    dictOfProposedstatus = {123: 'Stopped'}
    res = self.tc._applyTransformationStatusStateMachine(transIDAsDict, dictOfProposedstatus, False)
    self.assertEqual(res, 'New')

    transIDAsDict = {123: ['New', 'MCSimulation']}
    dictOfProposedstatus = {123: 'Stopped'}
    res = self.tc._applyTransformationStatusStateMachine(transIDAsDict, dictOfProposedstatus, True)
    self.assertEqual(res, 'Stopped')

    transIDAsDict = {123: ['New', 'MCSimulation']}
    dictOfProposedstatus = {123: 'Idle'}
    res = self.tc._applyTransformationStatusStateMachine(transIDAsDict, dictOfProposedstatus, False)
    self.assertEqual(res, 'New')

    transIDAsDict = {123: ['Active', 'MCSimulation']}
    dictOfProposedstatus = {123: 'Idle'}
    res = self.tc._applyTransformationStatusStateMachine(transIDAsDict, dictOfProposedstatus, False)
    self.assertEqual(res, 'Idle')

    transIDAsDict = {123: ['Active', 'MCSimulation']}
    dictOfProposedstatus = {123: 'Completed'}
    res = self.tc._applyTransformationStatusStateMachine(transIDAsDict, dictOfProposedstatus, False)
    self.assertEqual(res, 'Flush')

    transIDAsDict = {123: ['Active', 'MCSimulation']}
    dictOfProposedstatus = {123: 'Complete'}
    res = self.tc._applyTransformationStatusStateMachine(transIDAsDict, dictOfProposedstatus, False)
    self.assertEqual(res, 'Active')


# Test data for plugins
data = {'/this/is/at_1': ['SE1'],
        '/this/is/at_2': ['SE2'],
        '/this/is/at_12': ['SE1', 'SE2'],
        '/this/is/also/at_12': ['SE1', 'SE2'],
        '/this/is/at_123': ['SE1', 'SE2', 'SE3'],
        '/this/is/at_23': ['SE2', 'SE3'],
        '/this/is/at_4': ['SE4']}

cachedLFNSize = {'/this/is/at_1': 1,
                 '/this/is/at_2': 2,
                 '/this/is/at_12': 12,
                 '/this/is/also/at_12': 12,
                 '/this/is/at_123': 123,
                 '/this/is/at_23': 23,
                 '/this/is/at_4': 4}

runFiles = [{'LFN': '/this/is/at_1', 'RunNumber': 1},
            {'LFN': '/this/is/at_2', 'RunNumber': 2},
            {'LFN': '/this/is/at_12', 'RunNumber': 12},
            {'LFN': '/this/is/also/at_12', 'RunNumber': 12},
            {'LFN': '/this/is/at_123', 'RunNumber': 123},
            {'LFN': '/this/is/at_23', 'RunNumber': 23},
            {'LFN': '/this/is/at_4', 'RunNumber': 4}]

cachedLFNAncestors = {1: {'': 1}}


class PluginsUtilitiesSuccess(ClientTestCase):

  def test_groupByRun(self):

    # no files, nothing happens
    resDict = groupByRun([])
    self.assertEqual(resDict, {})

    # some files
    resDict = groupByRun(list(runFiles))
    resExpected = {1: ['/this/is/at_1'],
                   2: ['/this/is/at_2'],
                   4: ['/this/is/at_4'],
                   12: ['/this/is/at_12', '/this/is/also/at_12'],
                   23: ['/this/is/at_23'],
                   123: ['/this/is/at_123']}
    self.assertEqual(resDict, resExpected)

  def test_groupByRunAndParam(self):

    # no files, nothing happens
    pu = PluginUtilities(fc=self.fcMock, dataManager=MagicMock(), rmClient=MagicMock())
    pu.transFiles = []
    resDict = pu.getFilesGroupedByRunAndParam({})
    self.assertEqual(resDict, {})

    # some files, no params (it seems it's never called with params...?
    pu = PluginUtilities(fc=self.fcMock, dataManager=MagicMock(), rmClient=MagicMock())
    pu.transFiles = runFiles
    resDict = pu.getFilesGroupedByRunAndParam(data)
    resExpected = {1: {None: ['/this/is/at_1']},
                   2: {None: ['/this/is/at_2']},
                   4: {None: ['/this/is/at_4']},
                   12: {None: ['/this/is/at_12', '/this/is/also/at_12']},
                   23: {None: ['/this/is/at_23']},
                   123: {None: ['/this/is/at_123']}}
    self.assertEqual(resDict, resExpected)

  def test_getRAWAncestorsForRun(self):

    # no files, nothing happens
    #     tsMock = MagicMock()
    #     tsMock.getTransformationFiles.return_value=S_OK( [] )
    #     pu = PluginUtilities(transClient=tsMock, fc=self.fcMock, dataManager=MagicMock(), rmClient=MagicMock() )
    #     res = pu.getRAWAncestorsForRun( 1 )
    #     self.assertEqual( res, 0 )

    # some files
    #     tsMock = MagicMock()
    #     tsMock.getTransformationFiles.return_value = S_OK( [{'LFN':'this/is/at_1', 'Status':'Unused'},
    #                                                         {'LFN':'this/is/not_here', 'Status':'MissingInFC'}] )
    #     bkMock = MagicMock()
    #     bkMock.getFileAncestors.return_value =
    #     pu = PluginUtilities( transClient=tsMock, fc=self.fcMock, dataManager=MagicMock(), rmClient=MagicMock() )
    #     res = pu.getRAWAncestorsForRun( 1 )
    #     self.assertEqual( res, 0 )
    pass


#############################################################################
#############################################################################
if __name__ == '__main__':
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(ClientTestCase)
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TaskManagerSuccess))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TransformationClientSuccess))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(PluginsUtilitiesSuccess))
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript


def getBoolean(value):
  if value.lower() == 'true':
    return True
  elif value.lower() == 'false':
    return False
  else:
    print('ERROR: expected boolean')
    DIRAC.exit(2)


def usage():
  """usage.

  Prints script usage
  """
  from DIRAC.Core.Base import Script
  print('Usage: %s <ProductionID> <Options> [Try -h,--help for more information]' % Script.scriptName)
  DIRAC.exit(2)


@DIRACScript()
def main():
  from DIRAC.Core.Base import Script

  # Default values
  status = None
  outFile = None
  summary = True
  printVerbose = False

  Script.registerSwitch("", "Status=", "ProductionDB file status to select")
  Script.registerSwitch("", "OutputFile=", "Output file to store file records")
  Script.registerSwitch("", "Summary=", "Print a summary of the files (True/False) default is True")
  Script.registerSwitch("", "PrintOutput=", "Print all file records (extremely verbose) default is False")
  Script.parseCommandLine(ignoreErrors=True)

  from LHCbDIRAC.Interfaces.API.DiracProduction import DiracProduction

  for switch in Script.getUnprocessedSwitches():
    if switch[0].lower() == "status":
      status = switch[1]
    elif switch[0].lower() == "outputfile":
      outFile = switch[1]
    elif switch[0].lower() == "summary":
      summary = getBoolean(switch[1])
    elif switch[0].lower() == "printoutput":
      printVerbose = getBoolean(switch[1])

  args = Script.getPositionalArgs()

  if len(args) != 1:
    usage()

  exitCode = 0

  diracProd = DiracProduction()

  productionID = args[0]
  try:
    productionID = int(productionID)
  except Exception as x:
    print('Production ID must be an integer, not %s:\n%s' % (productionID, x))
    DIRAC.exit(2)

  result = diracProd.productionFileSummary(
      productionID,
      selectStatus=status,
      outputFile=outFile,
      printSummary=summary,
      printOutput=printVerbose,
  )
  if not result['OK']:
    print('ERROR %s' % result['Message'])
    exitCode = 2

  DIRAC.exit(exitCode)


if __name__ == "__main__":
  main()



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""It returns the input and output files of a given list of DIRAC Jobids."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import os
from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from DIRAC.Core.Base import Script
  from LHCbDIRAC.DataManagementSystem.Client.DMScript import DMScript, printDMResult

  bkScript = DMScript()
  bkScript.registerJobsSwitches()
  Script.registerSwitch('', 'InputFiles', '  Only input files')
  Script.registerSwitch('', 'OutputFiles', '  Only output files')
  Script.setUsageMessage(__doc__ + '\n'.join([
      'Usage:',
      '  %s [option|cfgfile] ... [DIRACJobid|File]' % Script.scriptName,
      'Arguments:',
      '  DIRACJobid:      DIRAC Jobids',
      '  File:     Name of the file with contains a list of DIRACJobids']))

  Script.parseCommandLine(ignoreErrors=True)
  args = Script.getPositionalArgs()
  inputFiles = False
  outputFiles = False
  for switch in Script.getUnprocessedSwitches():
    if switch[0] == 'InputFiles':
      inputFiles = True
    if switch[0] == 'OutputFiles':
      outputFiles = True
  if not inputFiles and not outputFiles:
    inputFiles = True
    outputFiles = True
  jobidList = []
  for jobid in args:
    if os.path.exists(jobid):
      bkScript.setJobidsFromFile(jobid)
    else:
      jobidList += jobid.split(',')
  jobidList += bkScript.getOption('JobIDs', [])
  if not jobidList:
    print("No jobID provided!")
    Script.showHelp(exitCode=1)

  from LHCbDIRAC.BookkeepingSystem.Client.BookkeepingClient import BookkeepingClient
  retVal = BookkeepingClient().getJobInputOutputFiles(jobidList)
  if retVal['OK']:
    success = retVal['Value']['Successful']
    for job in success:
      # Remove from input the files that are also output!
      # This happens because the output of step 1 can be the input of step 2...
      # only worth if input files are requested though
      if inputFiles:
        success[job]['InputFiles'] = sorted(set(success[job]['InputFiles']) - set(success[job]['OutputFiles']))

      if not inputFiles or not outputFiles:
        success[job].pop('InputFiles' if not inputFiles else 'OutputFiles')

  printDMResult(retVal, empty="File does not exists in the Bookkeeping")


if __name__ == "__main__":
  main()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Unit tests for LHCbDIRAC utilities."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# pylint: disable=protected-access,missing-docstring,invalid-name

__RCSID__ = "$Id$"

import unittest
import itertools
import os
import datetime

from mock import MagicMock

from DIRAC import gLogger

from LHCbDIRAC.BookkeepingSystem.Client.test.mock_BookkeepingClient import bkc_mock

from LHCbDIRAC.Core.Utilities.ProductionData import _makeProductionLFN, constructProductionLFNs, \
    _getLFNRoot, _applyMask, getLogPath, constructUserLFNs
from LHCbDIRAC.Core.Utilities.InputDataResolution import InputDataResolution
from LHCbDIRAC.Core.Utilities.ProdConf import ProdConf
from LHCbDIRAC.Core.Utilities.GangaDataFile import GangaDataFile
from LHCbDIRAC.Core.Utilities.NagiosConnector import NagiosConnector

gConfigMock = MagicMock()
gConfigMock.getValue.return_value = 'aValue'


class UtilitiesTestCase(unittest.TestCase):
  """Base class for the Utilities test cases."""

  def setUp(self):

    self.IDR = InputDataResolution({}, bkc_mock)
    self.pc = ProdConf()

    gLogger.setLevel('DEBUG')

  def tearDown(self):
    for fileProd in ['prodConf.py', 'data.py', 'gaudi_extra_options.py']:
      try:
        os.remove(fileProd)
      except OSError:
        continue

#################################################


class ProdConfSuccess(UtilitiesTestCase):

  def test__buildOptions(self):
    ret = self.pc._buildOptions({'Application': 'DaVinci', 'InputFiles': ['foo', 'bar']})
    self.assertEqual(ret, {'Application': 'DaVinci', 'InputFiles': ['foo', 'bar']})

    self.pc.whatsIn = {'AppVersion': 'v30r0'}
    ret = self.pc._buildOptions({'Application': 'DaVinci', 'InputFiles': ['foo', 'bar']})
    self.assertEqual(ret, {'Application': 'DaVinci', 'InputFiles': ['foo', 'bar'], 'AppVersion': 'v30r0'})

    ret = self.pc._buildOptions({'AppVersion': 'v31r0', 'InputFiles': ['foo', 'bar']})
    self.assertEqual(ret, {'InputFiles': ['foo', 'bar'], 'AppVersion': 'v31r0'})

  def test__getOptionsString(self):
    ret = self.pc._getOptionsString({'Application': 'DaVinci', 'InputFiles': ['foo', 'bar']})
    self.assertEqual(
        ret,
        "from ProdConf import ProdConf\n\nProdConf(\n  Application='DaVinci',\n  InputFiles=['foo', 'bar'],\n)")

  def test_complete(self):
    try:
      os.remove('prodConf.py')
    except OSError:
      pass
    pc1 = ProdConf()
    self.assertEqual(pc1.whatsIn, {})
    pc1.putOptionsIn({'Application': 'DaVinci'})
    self.assertEqual(pc1.whatsIn, {'Application': 'DaVinci'})

    try:
      os.remove('prodConf.py')
    except OSError:
      pass
    pc1 = ProdConf()
    pc1.putOptionsIn({'InputFiles': ['foo', 'bar']})
    self.assertEqual(pc1.whatsIn, {'InputFiles': ['foo', 'bar']})

    try:
      os.remove('prodConf.py')
    except OSError:
      pass
    pc1 = ProdConf()
    pc1.putOptionsIn({'RunNumber': 12345})
    self.assertEqual(pc1.whatsIn, {'RunNumber': 12345})
    fopen = open('prodConf.py', 'r')
    fileString = fopen.read()
    fopen.close()
    string = "from ProdConf import ProdConf\n\nProdConf(\n  RunNumber=12345,\n)"
    self.assertEqual(string, fileString)

    try:
      os.remove('prodConf.py')
    except OSError:
      pass
    pc1 = ProdConf()
    pc1.putOptionsIn({'Application': 'DaVinci', 'InputFiles': ['foo', 'bar']})
    self.assertEqual(pc1.whatsIn, {'Application': 'DaVinci', 'InputFiles': ['foo', 'bar']})

    try:
      os.remove('prodConf.py')
    except OSError:
      pass
    pc1 = ProdConf()
    pc1.putOptionsIn({'Application': 'DaVinci'})
    self.assertEqual(pc1.whatsIn, {'Application': 'DaVinci'})
    pc1.putOptionsIn({'Application': 'LHCb'})
    self.assertEqual(pc1.whatsIn, {'Application': 'LHCb'})

    try:
      os.remove('prodConf.py')
    except OSError:
      pass
    pc1 = ProdConf()
    pc1.putOptionsIn({'Application': 'DaVinci', 'InputFiles': ['foo', 'bar']})
    self.assertEqual(pc1.whatsIn, {'Application': 'DaVinci', 'InputFiles': ['foo', 'bar']})
    pc1.putOptionsIn({'Application': 'LHCb'})
    self.assertEqual(pc1.whatsIn, {'Application': 'LHCb', 'InputFiles': ['foo', 'bar']})
    pc1.putOptionsIn({'RunNumber': 12345})
    self.assertEqual(pc1.whatsIn, {'Application': 'LHCb', 'InputFiles': ['foo', 'bar'], 'RunNumber': 12345})

    try:
      os.remove('prodConf.py')
    except OSError:
      pass
    pc1 = ProdConf()

    pc1.putOptionsIn({'Application': 'DaVinci', 'InputFiles': ['foo', 'bar'], 'AppVersion': 'v30r0'})
    self.assertEqual(pc1.whatsIn, {'Application': 'DaVinci', 'InputFiles': ['foo', 'bar'], 'AppVersion': 'v30r0'})
    fopen = open('prodConf.py', 'r')
    fileString = fopen.read()
    fopen.close()
    string = "from ProdConf import ProdConf\n\nProdConf(\n  Application='DaVinci',\n  "
    string += "InputFiles=['foo', 'bar'],\n  AppVersion='v30r0',\n)"
    self.assertEqual(string, fileString)

    pc1.putOptionsIn({'Application': 'LHCb'})
    self.assertEqual(pc1.whatsIn, {'Application': 'LHCb', 'InputFiles': ['foo', 'bar'], 'AppVersion': 'v30r0'})
    fopen = open('prodConf.py', 'r')
    fileString = fopen.read()
    fopen.close()
    string = "from ProdConf import ProdConf\n\nProdConf(\n  Application='LHCb',\n  "
    string += "InputFiles=['foo', 'bar'],\n  AppVersion='v30r0',\n)"
    self.assertEqual(string, fileString)

    pc1.putOptionsIn({'InputFiles': ['pippo', 'pluto']})
    self.assertEqual(pc1.whatsIn, {'Application': 'LHCb', 'InputFiles': ['pippo', 'pluto'], 'AppVersion': 'v30r0'})
    fopen = open('prodConf.py', 'r')
    fileString = fopen.read()
    fopen.close()
    string = "from ProdConf import ProdConf\n\nProdConf(\n  Application='LHCb',\n  "
    string += "InputFiles=['pippo', 'pluto'],\n  AppVersion='v30r0',\n)"
    self.assertEqual(set(string.split()), set(fileString.split()))

    pc1.putOptionsIn({'InputFiles': []})
    self.assertEqual(pc1.whatsIn, {'Application': 'LHCb', 'InputFiles': [], 'AppVersion': 'v30r0'})
    fopen = open('prodConf.py', 'r')
    fileString = fopen.read()
    fopen.close()
    string = "from ProdConf import ProdConf\n\nProdConf(\n  Application='LHCb',\n  "
    string += "InputFiles=[],\n  AppVersion='v30r0',\n)"
    self.assertEqual(set(string.split()), set(fileString.split()))

    pc1.putOptionsIn({'Application': '', 'RunNumber': 12345})
    self.assertEqual(pc1.whatsIn, {'Application': '', 'InputFiles': [], 'AppVersion': 'v30r0', 'RunNumber': 12345})
    fopen = open('prodConf.py', 'r')
    fileString = fopen.read()
    fopen.close()
    string = "from ProdConf import ProdConf\n\nProdConf(\n  Application='',\n  "
    string += "InputFiles=[],\n  AppVersion='v30r0',\n  RunNumber=12345,\n)"
    self.assertEqual(set(string.split()), set(fileString.split()))


#################################################

class GangaDataFileSuccess(UtilitiesTestCase):

  def test_generateGangaDataFile(self):

    gdf = GangaDataFile()

    res = gdf.generateDataFile(['foo', 'bar'], persistency='ROOT')
    # Remove first line as it contains the date and time
    res = '\n'.join(res.split('\n')[1:])
    root = '\n' + \
        'from Gaudi.Configuration import * \n' + \
        "from GaudiConf import IOHelper\nIOHelper('ROOT').inputFiles([\n" + \
        "'LFN:foo',\n'LFN:bar',\n], clear=True)\n" + \
        "\nFileCatalog().Catalogs += [ 'xmlcatalog_file:pool_xml_catalog.xml' ]\n"
    self.assertEqual(res, root)

    res = gdf.generateDataFile(['foo', 'bar'])
    # Remove first line as it contains the date and time
    res = '\n'.join(res.split('\n')[1:])
    root = '\n' + \
        'from Gaudi.Configuration import * \n' + \
        "from GaudiConf import IOHelper\nIOHelper().inputFiles([\n" + \
        "'LFN:foo',\n'LFN:bar',\n], clear=True)\n" + \
        "\nFileCatalog().Catalogs += [ 'xmlcatalog_file:pool_xml_catalog.xml' ]\n"
    self.assertEqual(res, root)

    gdf = GangaDataFile(xmlcatalog_file='')

    res = gdf.generateDataFile(['foo', 'bar'], persistency='MDF')
    # Remove first line as it contains the date and time
    res = '\n'.join(res.split('\n')[1:])
    root = '\n' + \
        'from Gaudi.Configuration import * \n' + \
        "from GaudiConf import IOHelper\nIOHelper('MDF').inputFiles([\n" + \
        "'LFN:foo',\n'LFN:bar',\n], clear=True)\n"
    self.assertEqual(res, root)

    res = gdf.generateDataFile('foo', persistency='ROOT')
    # Remove first line as it contains the date and time
    res = '\n'.join(res.split('\n')[1:])
    root = '\n' + \
        'from Gaudi.Configuration import * \n' + \
        "from GaudiConf import IOHelper\nIOHelper('ROOT').inputFiles([\n" + \
        "'LFN:foo',\n], clear=True)\n"
    self.assertEqual(res, root)

#################################################


class GangaDataFileFailure(UtilitiesTestCase):

  def test_generateDataFile(self):

    gdf = GangaDataFile()
    self.assertRaises(TypeError, gdf.generateDataFile, (''))
    self.assertRaises(ValueError, gdf.generateDataFile, ([]))

#################################################


class ProductionDataSuccess(UtilitiesTestCase):

  #################################################

  def test_constructProductionLFNs(self):

    # + test with InputData

    paramDict = {'PRODUCTION_ID': '12345',
                 'JOB_ID': '54321',
                 'configVersion': 'test',
                 'configName': 'certification',
                 'JobType': 'MCSimulation',
                 'outputList': [{'outputDataType': 'sim',
                                 'outputDataSE': 'Tier1-RDST',
                                 'outputDataName': '00012345_00054321_1.sim'},
                                {'outputDataType': 'digi',
                                 'outputDataSE': 'Tier1-RDST',
                                 'outputDataName': '00012345_00054321_2.digi'},
                                {'outputDataType': 'dst',
                                 'outputDataSE': 'Tier1_MC_M-DST',
                                 'outputDataName': '00012345_00054321_4.dst'},
                                {'outputDataType': 'ALLSTREAMS.DST',
                                 'outputBKType': 'ALLSTREAMS.DST',
                                 'outputDataSE': 'Tier1_MC_M-DST',
                                 'outputDataName': '00012345_00054321_5.AllStreams.dst'}],
                 }

    AllStreamsDST = '/lhcb/certification/test/ALLSTREAMS.DST/00012345/0005/00012345_00054321_5.AllStreams.dst'
    reslist = [{'LogTargetPath': ['/lhcb/certification/test/LOG/00012345/0005/00012345_00054321.tar'],
                'LogFilePath': ['/lhcb/certification/test/LOG/00012345/0005/00054321'],
                'DebugLFNs': ['/lhcb/debug/test/SIM/00012345/0005/00012345_00054321_1.sim',
                              '/lhcb/debug/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                              '/lhcb/debug/test/DST/00012345/0005/00012345_00054321_4.dst',
                              '/lhcb/debug/test/ALLSTREAMS.DST/00012345/0005/00012345_00054321_5.AllStreams.dst',
                              '/lhcb/debug/test/CORE/00012345/0005/00054321_core'],
                'BookkeepingLFNs': ['/lhcb/certification/test/SIM/00012345/0005/00012345_00054321_1.sim',
                                    '/lhcb/certification/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                                    '/lhcb/certification/test/DST/00012345/0005/00012345_00054321_4.dst',
                                    AllStreamsDST],
                'ProductionOutputData': ['/lhcb/certification/test/SIM/00012345/0005/00012345_00054321_1.sim',
                                         '/lhcb/certification/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                                         '/lhcb/certification/test/DST/00012345/0005/00012345_00054321_4.dst',
                                         AllStreamsDST]},
               {'LogTargetPath': ['/lhcb/certification/test/LOG/00012345/0005/00012345_00054321.tar'],
                'LogFilePath': ['/lhcb/certification/test/LOG/00012345/0005/00054321'],
                'DebugLFNs': ['/lhcb/debug/test/SIM/00012345/0005/00012345_00054321_1.sim',
                              '/lhcb/debug/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                              '/lhcb/debug/test/DST/00012345/0005/00012345_00054321_4.dst',
                              '/lhcb/debug/test/ALLSTREAMS.DST/00012345/0005/00012345_00054321_5.AllStreams.dst',
                              '/lhcb/debug/test/CORE/00012345/0005/00054321_core'],
                'BookkeepingLFNs': ['/lhcb/certification/test/SIM/00012345/0005/00012345_00054321_1.sim',
                                    '/lhcb/certification/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                                    '/lhcb/certification/test/DST/00012345/0005/00012345_00054321_4.dst',
                                    AllStreamsDST],
                'ProductionOutputData': ['/lhcb/certification/test/DST/00012345/0005/00012345_00054321_4.dst']},
               {'LogTargetPath': ['/lhcb/certification/test/LOG/00012345/0005/00012345_00054321.tar'],
                'LogFilePath': ['/lhcb/certification/test/LOG/00012345/0005/00054321'],
                'DebugLFNs': ['/lhcb/debug/test/SIM/00012345/0005/00012345_00054321_1.sim',
                              '/lhcb/debug/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                              '/lhcb/debug/test/DST/00012345/0005/00012345_00054321_4.dst',
                              '/lhcb/debug/test/ALLSTREAMS.DST/00012345/0005/00012345_00054321_5.AllStreams.dst',
                              '/lhcb/debug/test/CORE/00012345/0005/00054321_core'],
                'BookkeepingLFNs': ['/lhcb/certification/test/SIM/00012345/0005/00012345_00054321_1.sim',
                                    '/lhcb/certification/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                                    '/lhcb/certification/test/DST/00012345/0005/00012345_00054321_4.dst',
                                    AllStreamsDST],
                'ProductionOutputData': ['/lhcb/certification/test/DST/00012345/0005/00012345_00054321_4.dst']},
               {'LogTargetPath': ['/lhcb/certification/test/LOG/00012345/0005/00012345_00054321.tar'],
                'LogFilePath': ['/lhcb/certification/test/LOG/00012345/0005/00054321'],
                'DebugLFNs': ['/lhcb/debug/test/SIM/00012345/0005/00012345_00054321_1.sim',
                              '/lhcb/debug/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                              '/lhcb/debug/test/DST/00012345/0005/00012345_00054321_4.dst',
                              '/lhcb/debug/test/ALLSTREAMS.DST/00012345/0005/00012345_00054321_5.AllStreams.dst',
                              '/lhcb/debug/test/CORE/00012345/0005/00054321_core'],
                'BookkeepingLFNs': ['/lhcb/certification/test/SIM/00012345/0005/00012345_00054321_1.sim',
                                    '/lhcb/certification/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                                    '/lhcb/certification/test/DST/00012345/0005/00012345_00054321_4.dst',
                                    AllStreamsDST],
                'ProductionOutputData': ['/lhcb/certification/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                                         '/lhcb/certification/test/DST/00012345/0005/00012345_00054321_4.dst']},
               {'LogTargetPath': ['/lhcb/certification/test/LOG/00012345/0005/00012345_00054321.tar'],
                'LogFilePath': ['/lhcb/certification/test/LOG/00012345/0005/00054321'],
                'DebugLFNs': ['/lhcb/debug/test/SIM/00012345/0005/00012345_00054321_1.sim',
                              '/lhcb/debug/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                              '/lhcb/debug/test/DST/00012345/0005/00012345_00054321_4.dst',
                              '/lhcb/debug/test/ALLSTREAMS.DST/00012345/0005/00012345_00054321_5.AllStreams.dst',
                              '/lhcb/debug/test/CORE/00012345/0005/00054321_core'],
                'BookkeepingLFNs': ['/lhcb/certification/test/SIM/00012345/0005/00012345_00054321_1.sim',
                                    '/lhcb/certification/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                                    '/lhcb/certification/test/DST/00012345/0005/00012345_00054321_4.dst',
                                    AllStreamsDST],
                'ProductionOutputData': [AllStreamsDST]},
               {'LogTargetPath': ['/lhcb/certification/test/LOG/00012345/0005/00012345_00054321.tar'],
                'LogFilePath': ['/lhcb/certification/test/LOG/00012345/0005/00054321'],
                'DebugLFNs': ['/lhcb/debug/test/SIM/00012345/0005/00012345_00054321_1.sim',
                              '/lhcb/debug/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                              '/lhcb/debug/test/DST/00012345/0005/00012345_00054321_4.dst',
                              '/lhcb/debug/test/ALLSTREAMS.DST/00012345/0005/00012345_00054321_5.AllStreams.dst',
                              '/lhcb/debug/test/CORE/00012345/0005/00054321_core'],
                'BookkeepingLFNs': ['/lhcb/certification/test/SIM/00012345/0005/00012345_00054321_1.sim',
                                    '/lhcb/certification/test/DIGI/00012345/0005/00012345_00054321_2.digi',
                                    '/lhcb/certification/test/DST/00012345/0005/00012345_00054321_4.dst',
                                    AllStreamsDST],
                'ProductionOutputData': ['/lhcb/certification/test/DST/00012345/0005/00012345_00054321_4.dst',
                                         AllStreamsDST]},
               ]

    outputDataFileMasks = ('', 'dst', 'DST', ['digi', 'dst'], 'ALLSTREAMS.DST', ['dst', 'allstreams.dst'])

    for outputDataFileMask, resL in zip(outputDataFileMasks, reslist):
      paramDict['outputDataFileMask'] = outputDataFileMask

      res = constructProductionLFNs(paramDict, bkc_mock)

      self.assertTrue(res['OK'])
      self.assertEqual(res['Value'], resL)

      resWithBkk = constructProductionLFNs(paramDict, bkc_mock, quick=False)

      self.assertTrue(resWithBkk['OK'])
      self.assertEqual(resWithBkk['Value'], resL)

  #################################################

  def test__makeProductionLFN(self):

    JOB_ID = '00054321'
    LFN_ROOT = '/lhcb/MC/MC10'
    filetuple = (('00012345_00054321_1.sim', 'sim'), ('00012345_00054321_1.sim', 'SIM'))
    prodstring = '00012345'

    for ft in filetuple:
      res = _makeProductionLFN(JOB_ID, LFN_ROOT, ft, prodstring)
      self.assertEqual(res, '/lhcb/MC/MC10/SIM/00012345/0005/00012345_00054321_1.sim')

  #################################################

  def test__applymask(self):

    dtl = [('00012345_00054321_1.sim', 'sim'),
           ('00012345_00054321_4.dst', 'dst'),
           ('00012345_00054321_2.digi', 'digi'),
           ('Brunel_00012345_00012345_1_Hist.root', 'hist'),
           ('00012345_00054321_5.AllStreams.dst', 'ALLSTREAMS.DST')]

    wfMask = ('', 'dst', 'ALLSTREAMS.DST', ['dst', 'digi'], ['DIGI', 'allstreams.dst'], 'hist', ['dst', 'hist'])

    dtlM = ([('00012345_00054321_1.sim', 'sim'),
             ('00012345_00054321_4.dst', 'dst'),
             ('00012345_00054321_2.digi', 'digi'),
             ('Brunel_00012345_00012345_1_Hist.root', 'hist'),
             ('00012345_00054321_5.AllStreams.dst', 'ALLSTREAMS.DST')
             ],
            [('00012345_00054321_4.dst', 'dst')],
            [('00012345_00054321_5.AllStreams.dst', 'ALLSTREAMS.DST')],
            [('00012345_00054321_4.dst', 'dst'),
             ('00012345_00054321_2.digi', 'digi')
             ],
            [('00012345_00054321_2.digi', 'digi'),
             ('00012345_00054321_5.AllStreams.dst', 'ALLSTREAMS.DST')
             ],
            [('Brunel_00012345_00012345_1_Hist.root', 'hist')
             ],
            [('00012345_00054321_4.dst', 'dst'),
             ('Brunel_00012345_00012345_1_Hist.root', 'hist')
             ]
            )

    for mask, res in zip(wfMask, dtlM):
      r = _applyMask(mask, dtl)

      self.assertEqual(r, res)

  def test_getLogPath(self):

    wkf_commons = {'PRODUCTION_ID': 12345,
                   'JOB_ID': 0o0001,
                   'configName': 'LHCb',
                   'configVersion': 'Collision11',
                   'JobType': 'MCSimulation'}

    resWithBkk = getLogPath(wkf_commons, bkc_mock, quick=False)

    self.assertEqual(resWithBkk,
                     {'OK': True,
                      'Value': {'LogTargetPath': ['/lhcb/LHCb/Collision11/LOG/00012345/0000/00012345_00000001.tar'],
                                'LogFilePath': ['/lhcb/LHCb/Collision11/LOG/00012345/0000/00000001']}})

    res = getLogPath(wkf_commons, bkc_mock)

    self.assertEqual(res,
                     {'OK': True,
                      'Value': {'LogTargetPath': ['/lhcb/LHCb/Collision11/LOG/00012345/0000/00012345_00000001.tar'],
                                'LogFilePath': ['/lhcb/LHCb/Collision11/LOG/00012345/0000/00000001']}})

  def test__getLFNRoot(self):
    res = _getLFNRoot('/lhcb/data/CCRC08/00009909/DST/0000/00009909_00003456_2.dst', 'MC12', bkClient=bkc_mock)
    self.assertEqual(res, '/lhcb/data/CCRC08/00009909/DST/0000/00009909_00003456_2.dst')

    res = _getLFNRoot(
        '/lhcb/data/CCRC08/00009909/DST/0000/00009909_00003456_2.dst',
        'MC12',
        bkClient=bkc_mock,
        quick=True)
    self.assertEqual(res, '/lhcb/data/CCRC08/00009909')

  def test_constructUserLFNs(self):
    timeTup = datetime.date.today().timetuple()
    yearMonth = '%s_%s' % (timeTup[0], str(timeTup[1]).zfill(2))

    res = constructUserLFNs(123, 'fstagni', ['pippo.txt', 'pluto.txt'])
    self.assertEqual(sorted(res), sorted(['/lhcb/user/f/fstagni/' + yearMonth + '/0/123/pluto.txt',
                                          '/lhcb/user/f/fstagni/' + yearMonth + '/0/123/pippo.txt']))

    res = constructUserLFNs(123, 'fstagni', ['pippo.txt', 'pluto.txt'], 'my/Output/Path')
    self.assertEqual(sorted(res), sorted(['/lhcb/user/f/fstagni/my/Output/Path/' + yearMonth + '/0/123/pluto.txt',
                                          '/lhcb/user/f/fstagni/my/Output/Path/' + yearMonth + '/0/123/pippo.txt']))

    res = constructUserLFNs(123, 'fstagni', ['pippo.txt', 'pluto.txt'], prependString='pre/pend')
    self.assertEqual(sorted(res), sorted(['/lhcb/user/f/fstagni/' + yearMonth + '/123_pre/pend_pippo.txt',
                                          '/lhcb/user/f/fstagni/' + yearMonth + '/123_pre/pend_pluto.txt']))

    res = constructUserLFNs(123, 'fstagni', ['pippo.txt', 'pluto.txt'], 'my/Output/Path', 'pre/pend')
    self.assertEqual(sorted(res),
                     sorted(['/lhcb/user/f/fstagni/my/Output/Path/' + yearMonth + '/123_pre/pend_pippo.txt',
                             '/lhcb/user/f/fstagni/my/Output/Path/' + yearMonth + '/123_pre/pend_pluto.txt']))


class InputDataResolutionSuccess(UtilitiesTestCase):

  def test__addPfnType(self):

    res = self.IDR._addPfnType({'lfn1': {'mdata': 'mdata1'},
                                'lfn2': {'mdata': 'mdata2'}
                                })
    self.assertEqual(res, {'OK': True,
                           'Value': {'lfn1': {'pfntype': 'ROOT', 'mdata': 'mdata1'},
                                     'lfn2': {'pfntype': 'MDF', 'mdata': 'mdata2'}}
                           })

    bkc_mock.getFileTypeVersion.return_value = {'OK': True, 'Value': {}}

    res = self.IDR._addPfnType({'lfn1': {'mdata': 'mdata1'},
                                'lfn2': {'mdata': 'mdata2'}})
    self.assertEqual(res, {'OK': True,
                           'Value': {'lfn1': {'pfntype': 'ROOT', 'mdata': 'mdata1'},
                                     'lfn2': {'pfntype': 'ROOT', 'mdata': 'mdata2'}}
                           })

    bkc_mock.getFileTypeVersion.return_value = {'OK': True, 'Value': {'lfn1': 'ROOT'}}

    res = self.IDR._addPfnType({'lfn1': {'mdata': 'mdata1'},
                                'lfn2': {'mdata': 'mdata2'},
                                'lfn3': {'mdata': 'mdata3'}})
    self.assertEqual(res, {'OK': True,
                           'Value': {'lfn1': {'pfntype': 'ROOT', 'mdata': 'mdata1'},
                                     'lfn2': {'pfntype': 'ROOT', 'mdata': 'mdata2'},
                                     'lfn3': {'pfntype': 'ROOT', 'mdata': 'mdata3'}}
                           })


class NagiosConnectorSuccess(UtilitiesTestCase):

  def test_readConfig(self):

    nagConn = NagiosConnector()
    nagConn.readConfig()
    self.assertTrue(nagConn.config['MsgPort'])

#  def test_failedConnection( self ):
#
#    nagConn = NagiosConnector()
#    nagConn.readConfig()
#    nagConn.config['NagiosName'] = 'lhcb-Dirac.Unittest'
#    nagConn.useDebugMessage()
#    self.assertEqual( nagConn.initializeConnection(),
#                       S_OK( 'Connection to Broker established' ),
#                       'Connection not correctly initialized' )
#    self.assertEqual( nagConn.sendMessage(),
#                       S_OK('Message sent to Broker.'),
#                       'Sending unsuccessful!' )


#############################################################################
# Test Suite run
#############################################################################

if __name__ == '__main__':
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(UtilitiesTestCase)
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ProductionDataSuccess))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ProdConfSuccess))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(InputDataResolutionSuccess))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(NagiosConnectorSuccess))
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Moving toward a templates-less system.

The RecoStripping Template creates workflows for the following use-cases:
  WORKFLOW1: Reconstruction
  WORKFLOW2: Stripping+Merge
  WORKFLOW3: RecoStripping+Merge (Reco and Stripping within the same job)
  WORKFLOW4: Reconstruction+Stripping+Merge
  WORKFLOW5: Stripping+Merge+Indexing

Exotic things you might want to do:
* run a local test:
  pre: remember to check if your input file is online, if not use lcg-bringonline <PFN>
* run only part of the request on the Grid:
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from six.moves import range

import ast

from DIRAC.Core.Base import Script
Script.parseCommandLine()

from DIRAC import gConfig, gLogger, exit as DIRACexit
from LHCbDIRAC.ProductionManagementSystem.Client.ProductionRequest import ProductionRequest

__RCSID__ = "$Id$"

gLogger = gLogger.getSubLogger('RecoStripping_run.py')
currentSetup = gConfig.getValue('DIRAC/Setup')

pr = ProductionRequest()

stepsList = ['{{p1Step}}']
stepsList.append('{{p2Step}}')
stepsList.append('{{p3Step}}')
stepsList.append('{{p4Step}}')
stepsList.append('{{p5Step}}')
stepsList.append('{{p6Step}}')
stepsList.append('{{p7Step}}')
stepsList.append('{{p8Step}}')
stepsList.append('{{p9Step}}')
pr.stepsList = stepsList

###########################################
# Configurable and fixed parameters
###########################################

pr.appendName = '{{WorkflowAppendName#GENERAL: Workflow string to append to production name#1}}'

w = '{{w#----->WORKFLOW: choose one below#}}'
w1 = '{{w1#-WORKFLOW1: Reconstruction#False}}'
w2 = '{{w2#-WORKFLOW2: Stripping+Merge#False}}'
w3 = '{{w3#-WORKFLOW3: RecoStripping+Merge#False}}'
w4 = '{{w4#-WORKFLOW4: Reconstruction+Stripping+Merge#False}}'
w5 = '{{w5#-WORKFLOW5: Stripping+Merge+Indexing #False}}'

localTestFlag = '{{localTestFlag#GENERAL: Set True for local test#False}}'
validationFlag = '{{validationFlag#GENERAL: Set True for validation prod#False}}'

# workflow params for all productions
pr.startRun = '{{startRun#GENERAL: run start, to set the start run#0}}'
pr.endRun = '{{endRun#GENERAL: run end, to set the end of the range#0}}'
pr.derivedProduction = '{{AncestorProd#GENERAL: ancestor prod to be derived#0}}'

# reco params
recoPriority = int('{{RecoPriority#PROD-1:RECO(Stripp): priority#2}}')
recoCPU = '{{RecoMaxCPUTime#PROD-1:RECO(Stripp): Max CPU time in secs#1500000}}'
recoPlugin = '{{RecoPluginType#PROD-1:RECO(Stripp): production plugin name#RAWProcessing}}'
recoFilesPerJob = '{{RecoFilesPerJob#PROD-1:RECO(Stripp): Group size or number of files per job#1}}'
recoDataSE = '{{RecoDataSE#PROD-1:RECO(Stripp): Output Data Storage Element#Tier1-Buffer}}'
try:
  recoDataSESpecial = ast.literal_eval('{{RecoDataSESpecial#PROD-1:RECO(Stripp): Special SE (a dictionary Type:SE)#}}')
except SyntaxError:
  recoDataSESpecial = {}
recoAncestorDepth = int('{{recoAncestorDepth#PROD-1: Ancestor Depth#0}}')
recoCompressionLvl = '{{recoCompressionLvl#PROD-1: compression level#LOW}}'
recoOutputVisFlag = '{{recoOutputVisFlag#PROD-1: Visibility flag of output files #Y}}'
try:
  recoOutputVisFlagSpecial = ast.literal_eval(
      '{{recoOutputVisFlagSpecial#PROD-1: Special Visibility flag of output files (dict FType:Y|N )#}}')
except SyntaxError:
  recoOutputVisFlagSpecial = {}

# stripp params
strippPriority = int('{{priority#PROD-2:Stripping: priority#2}}')
strippCPU = '{{StrippMaxCPUTime#PROD-2:Stripping: Max CPU time in secs#1000000}}'
strippPlugin = '{{StrippPluginType#PROD-2:Stripping: plugin name#ByRunWithFlush}}'
strippFilesPerJob = '{{StrippFilesPerJob#PROD-2:Stripping: Group size or number of files per job#2}}'
strippDataSE = '{{StrippStreamSE#PROD-2:Stripping: output data SE (un-merged streams)#Tier1-Buffer}}'
try:
  strippDataSESpecial = ast.literal_eval('{{StrippDataSESpecial#PROD-2:Stripping: Special SE (a dictionary Type:SE)#}}')
except SyntaxError:
  strippDataSESpecial = {}
strippAncestorDepth = int('{{strippAncestorDepth#PROD-2: Ancestor Depth#0}}')
strippCompressionLvl = '{{strippCompressionLvl#PROD-2: compression level#LOW}}'
strippOutputVisFlag = '{{strippOutputVisFlag#PROD-2: Visibility flag of output files#N}}'
try:
  strippOutputVisFlagSpecial = ast.literal_eval(
      '{{strippOutputVisFlagSpecial#PROD-2: Special Visibility flag of output files (dict FType:Y|N)#}}')
except SyntaxError:
  strippOutputVisFlagSpecial = {}

# merging params
mergingPriority = int('{{MergePriority#PROD-3:Merging: priority#8}}')
mergingCPU = '{{MergeMaxCPUTime#PROD-3:Merging: Max CPU time in secs#300000}}'
mergingPlugin = '{{MergePlugin#PROD-3:Merging: plugin#ByRunFileTypeSizeWithFlush}}'
mergingGroupSize = '{{MergeFileSize#PROD-3:Merging: Size (in GB) of the merged files#5}}'
mergingDataSE = '{{MergeStreamSE#PROD-3:Merging: output data SE (merged streams)#Tier1-DST}}'
try:
  mergingDataSESpecial = ast.literal_eval('{{MergingDataSESpecial#PROD-3:Merging: Special SE (a dictionary Type:SE)#}}')
except SyntaxError:
  mergingDataSESpecial = {}
mergingRemoveInputsFlag = '{{MergeRemoveFlag#PROD-3:Merging: remove input data flag True/False#True}}'
mergeCompressionLvl = '{{mergeCompressionLvl#PROD-3: compression level#HIGH}}'
mergeOutputVisFlag = '{{mergeOutputVisFlag#PROD-3: Visibility flag of output files#Y}}'
try:
  mergeOutputVisFlagSpecial = ast.literal_eval(
      '{{mergeOutputVisFlagSpecial#PROD-3: Special Visibility flag of output files (dict FType:Y|N)#}}')
except SyntaxError:
  mergeOutputVisFlagSpecial = {}

# indexing params
indexingPriority = int('{{IndexingPriority#PROD-4:indexing: priority#5}}')
indexingCPU = '{{IndexingMaxCPUTime#PROD-4:indexing: Max CPU time in secs#50000}}'
indexingPlugin = '{{IndexingPlugin#PROD-4:indexing: plugin#ByRunFileTypeSizeWithFlush}}'
indexingGroupSize = '{{IndexingFileSize#PROD-4:indexing: Size (in GB) of the Indexed files#50}}'
indexingDataSE = '{{IndexingStreamSE#PROD-4:indexing: output data SE (Indexed streams)#IndexerSE}}'

pr.requestID = '{{ID}}'
pr.prodGroup = '{{inProPass}}' + '/' + '{{pDsc}}'
# used in case of a test e.g. certification etc.
pr.configName = '{{configName}}'
pr.configVersion = '{{configVersion}}'
# Other parameters from the request page
pr.dqFlag = '{{inDataQualityFlag}}'  # UNCHECKED
pr.dataTakingConditions = '{{simDesc}}'
pr.processingPass = '{{inProPass}}'
pr.bkFileType = '{{inFileType}}'
pr.eventType = '{{eventType}}'
pr.visibility = 'Yes'
recoType = 'DataReconstruction'
targetSite = 'ALL'
recoMulticoreFlag = strippMulticoreFlag = mergeMulticoreFlag = 'True'
recoIDPolicy = strippIDPolicy = mergingIDPolicy = indexingIDPolicy = 'download'


w1 = ast.literal_eval(w1)
w2 = ast.literal_eval(w2)
w3 = ast.literal_eval(w3)
w4 = ast.literal_eval(w4)
w5 = ast.literal_eval(w5)

localTestFlag = ast.literal_eval(localTestFlag)
validationFlag = ast.literal_eval(validationFlag)

mergeRemoveInputsFlag = ast.literal_eval(mergingRemoveInputsFlag)

if not w1 and not w2 and not w3 and not w4:
  gLogger.error('I told you to select at least one workflow!')
  DIRACexit(2)

if localTestFlag:
  pr.testFlag = True
  pr.publishFlag = False
  pr.prodsToLaunch = [1]

recoInputDataList = []
strippInputDataList = []
if not pr.publishFlag:
  # this is 1380Gev MagUp
  # recoTestData = 'LFN:/lhcb/data/2011/RAW/FULL/LHCb/COLLISION11/88162/088162_0000000020.raw'
  # this is collision11
  recoTestData = 'LFN:/lhcb/data/2011/RAW/FULL/LHCb/COLLISION11/89333/089333_0000000003.raw'
  recoInputDataList.append(recoTestData)
  recoIDPolicy = 'protocol'

  strippTestData = 'LFN:/lhcb/data/2010/SDST/00008375/0001/00008375_00016947_1.sdst'
  strippInputDataList.append(strippTestData)
#  strippTestDataRAW = 'LFN:/lhcb/data/2010/RAW/FULL/LHCb/COLLISION10/75338/075338_0000000069.raw'
#  strippInputDataList.append( strippTestDataRAW )
  strippIDPolicy = 'protocol'
  evtsPerJob = '2000'


pr.outConfigName = pr.configName

# In case we want just to test, we publish in the certification/test part of the BKK
if currentSetup == 'LHCb-Certification' or pr.testFlag:
  pr.outConfigName = 'certification'
  if 'lhcb' not in pr.configName.lower():  # in case of reconstruction
    pr.configVersion = 'test'

if pr.testFlag:
  pr.dataTakingConditions = 'Beam3500GeV-VeloClosed-MagUp'
  if w1 or w3:
    pr.events = ['25']
    pr.processingPass = 'Real Data'
    pr.bkFileType = 'RAW'
  else:
    pr.events = ['2000']
    pr.processingPass = 'Real Data/Reco12'
    pr.bkFileType = 'SDST'
  mergingGroupSize = '1'
  recoCPU = strippCPU = '200000'
  pr.startRun = '93718'
  pr.endRun = '93720'
  pr.eventType = '90000000'
  pr.dqFlag = 'ALL'

if validationFlag:
  pr.outConfigName = 'validation'

if w1:
  pr.prodsTypeList = [recoType]
  pr.outputSEs = [recoDataSE]
  pr.specialOutputSEs = [recoDataSESpecial]
  pr.stepsInProds = [list(range(1, len(pr.stepsList) + 1))]
  pr.removeInputsFlags = [False]
  pr.priorities = [recoPriority]
  pr.cpus = [recoCPU]
  pr.groupSizes = [recoFilesPerJob]
  pr.plugins = [recoPlugin]
  pr.inputs = [recoInputDataList]
  pr.inputDataPolicies = [recoIDPolicy]
  pr.bkQueries = ['Full']
  pr.targets = [targetSite]
  pr.multicore = [recoMulticoreFlag]
  pr.outputModes = ['Run']
  pr.ancestorDepths = [recoAncestorDepth]
  pr.compressionLvl = [recoCompressionLvl] * len(pr.stepsInProds[0])
  pr.outputVisFlag = [{str(i + 1): recoOutputVisFlag} for i in range(len(pr.stepsInProds[0]))]
  pr.specialOutputVisFlag = [{"1": recoOutputVisFlagSpecial}]

elif w2:
  pr.prodsTypeList = ['DataStripping', 'Merge']
  pr.outputSEs = [strippDataSE, mergingDataSE]
  pr.specialOutputSEs = [strippDataSESpecial, mergingDataSESpecial]
  pr.stepsInProds = [list(range(1, len(pr.stepsList))),
                     [len(pr.stepsList)]]
  pr.removeInputsFlags = [False, mergeRemoveInputsFlag]
  pr.priorities = [strippPriority, mergingPriority]
  pr.cpus = [strippCPU, mergingCPU]
  pr.groupSizes = [strippFilesPerJob, mergingGroupSize]
  pr.plugins = [strippPlugin, mergingPlugin]
  pr.inputs = [strippInputDataList, []]
  pr.inputDataPolicies = [strippIDPolicy, mergingIDPolicy]
  pr.bkQueries = ['Full', 'fromPreviousProd']
  pr.targets = [targetSite, targetSite]
  pr.multicore = [strippMulticoreFlag, mergeMulticoreFlag]
  pr.outputModes = ['Run', 'Run']
  pr.ancestorDepths = [strippAncestorDepth, 0]
  pr.compressionLvl = [strippCompressionLvl] * len(pr.stepsInProds[0]) + \
                      [mergeCompressionLvl] * len(pr.stepsInProds[1])
  pr.outputVisFlag = [{"1": strippOutputVisFlag}, {"2": mergeOutputVisFlag}]
  pr.specialOutputVisFlag = [{"1": strippOutputVisFlagSpecial}, {"2": mergeOutputVisFlagSpecial}]

elif w3:
  pr.prodsTypeList = [recoType, 'Merge']
  pr.outputSEs = [recoDataSE, mergingDataSE]
  pr.specialOutputSEs = [recoDataSESpecial, mergingDataSESpecial]
  pr.stepsInProds = [list(range(1, len(pr.stepsList))),
                     [len(pr.stepsList)]]
  pr.removeInputsFlags = [False, mergeRemoveInputsFlag]
  pr.priorities = [recoPriority, mergingPriority]
  pr.cpus = [recoCPU, mergingCPU]
  pr.groupSizes = [recoFilesPerJob, mergingGroupSize]
  pr.plugins = [recoPlugin, mergingPlugin]
  pr.inputs = [recoInputDataList, []]
  pr.inputDataPolicies = [recoIDPolicy, mergingIDPolicy]
  pr.bkQueries = ['Full', 'fromPreviousProd']
  pr.targets = [targetSite, targetSite]
  pr.multicore = [recoMulticoreFlag, mergeMulticoreFlag]
  pr.outputModes = ['Run', 'Run']
  pr.ancestorDepths = [recoAncestorDepth, 0]
  pr.compressionLvl = [recoCompressionLvl] * len(pr.stepsInProds[0]) + \
                      [mergeCompressionLvl] * len(pr.stepsInProds[1])
  pr.outputVisFlag = [{"1": recoOutputVisFlag}, {"2": mergeOutputVisFlag}]
  pr.specialOutputVisFlag = [{"1": recoOutputVisFlagSpecial}, {"2": mergeOutputVisFlagSpecial}]

elif w4:
  pr.prodsTypeList = [recoType, 'DataStripping', 'Merge']
  pr.outputSEs = [recoDataSE, strippDataSE, mergingDataSE]
  pr.specialOutputSEs = [recoDataSESpecial, strippDataSESpecial, mergingDataSESpecial]
  pr.stepsInProds = [list(range(1, len(pr.stepsList) - 1)),
                     list(range(len(pr.stepsList) - 1, len(pr.stepsList))),
                     [len(pr.stepsList)]]
  pr.removeInputsFlags = [False, False, mergeRemoveInputsFlag]
  pr.priorities = [recoPriority, strippPriority, mergingPriority]
  pr.cpus = [recoCPU, strippCPU, mergingCPU]
  pr.groupSizes = [recoFilesPerJob, strippFilesPerJob, mergingGroupSize]
  pr.plugins = [recoPlugin, strippPlugin, mergingPlugin]
  pr.inputs = [recoInputDataList, [], []]
  pr.inputDataPolicies = [recoIDPolicy, strippIDPolicy, mergingIDPolicy]
  pr.bkQueries = ['Full', 'fromPreviousProd', 'fromPreviousProd']
  pr.targets = [targetSite, targetSite, targetSite]
  pr.multicore = [recoMulticoreFlag, strippMulticoreFlag, mergeMulticoreFlag]
  pr.outputModes = ['Run', 'Run', 'Run']
  pr.ancestorDepths = [recoAncestorDepth, strippAncestorDepth, 0]
  pr.compressionLvl = [recoCompressionLvl] * len(pr.stepsInProds[0]) + \
                      [strippCompressionLvl] * len(pr.stepsInProds[1]) + \
                      [mergeCompressionLvl] * len(pr.stepsInProds[2])
  pr.outputVisFlag = [{"1": recoOutputVisFlag}, {"2": strippOutputVisFlag}, {"3": mergeOutputVisFlag}]
  pr.specialOutputVisFlag = [{"1": recoOutputVisFlagSpecial},
                             {"2": strippOutputVisFlagSpecial}, {"3": mergeOutputVisFlagSpecial}]

elif w5:
  pr.prodsTypeList = ['DataStripping', 'Merge', 'WGProduction']
  pr.outputSEs = [strippDataSE, mergingDataSE, indexingDataSE]
  pr.specialOutputSEs = [strippDataSESpecial, mergingDataSESpecial, {}]
  pr.stepsInProds = [list(range(1, len(pr.stepsList) - 1)),
                     list(range(len(pr.stepsList) - 1, len(pr.stepsList))),
                     [len(pr.stepsList)]]
  pr.removeInputsFlags = [False, mergeRemoveInputsFlag, False]
  pr.priorities = [strippPriority, mergingPriority, indexingPriority]
  pr.cpus = [strippCPU, mergingCPU, indexingCPU]
  pr.groupSizes = [strippFilesPerJob, mergingGroupSize, indexingGroupSize]
  pr.plugins = [strippPlugin, mergingPlugin, indexingPlugin]
  pr.inputs = [strippInputDataList, [], []]
  pr.inputDataPolicies = [strippIDPolicy, mergingIDPolicy, indexingIDPolicy]
  pr.bkQueries = ['Full', 'fromPreviousProd', 'fromPreviousProd']
  pr.targets = [targetSite, targetSite, targetSite]
  pr.multicore = [strippMulticoreFlag, mergeMulticoreFlag, False]
  pr.outputModes = ['Run', 'Run', 'Any']
  pr.ancestorDepths = [strippAncestorDepth, 0, 0]
  pr.compressionLvl = [strippCompressionLvl] * len(pr.stepsInProds[0]) + \
                      [mergeCompressionLvl] * len(pr.stepsInProds[1]) + \
                      [''] * len(pr.stepsInProds[2])
  pr.outputVisFlag = [{"1": strippOutputVisFlag}, {"2": mergeOutputVisFlag}]
  pr.specialOutputVisFlag = [{"1": strippOutputVisFlagSpecial}, {"2": mergeOutputVisFlagSpecial}]


pr.buildAndLaunchRequest()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""RAWIntegrityDB is the front end for the database containing the files which
are awating migration.

It offers a simple interface to add files, get files and modify their
status.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import six

from DIRAC import gLogger, S_OK, S_ERROR
from DIRAC.Core.Base.DB import DB
from DIRAC.Core.Utilities.List import stringListToString
gLogger.initialize('DMS', '/Databases/RAWIntegrityDB/Test')


class RAWIntegrityDB(DB):

  _tablesDict = {}
  # Files table
  _tablesDict['Files'] = {
      'Fields': {
          'ID': 'BIGINT NOT NULL AUTO_INCREMENT',
          'LFN': 'VARCHAR(255) NOT NULL',
          'PFN': 'VARCHAR(255) NOT NULL',
          'Size': 'BIGINT NOT NULL',
          'StorageElement': 'VARCHAR(32) NOT NULL',
          'GUID': 'VARCHAR(255) NOT NULL',
          'FileChecksum': 'VARCHAR(255) NOT NULL',
          'SubmitTime': 'DATETIME NOT NULL',
          'CompleteTime': 'DATETIME',
          'Status': 'VARCHAR(255) DEFAULT "Active"'
      },
      'PrimaryKey': 'ID',
      'Indexes': {
          'Status': ['Status'],
          'LFN': ['LFN']
      }
  }
  # LastMonitor table
  _tablesDict['LastMonitor'] = {'Fields': {'LastMonitorTime': 'DATETIME NOT NULL PRIMARY KEY'}}

  def __init__(self, systemInstance='Default'):
    DB.__init__(self, 'RAWIntegrityDB', 'DataManagement/RAWIntegrityDB')

  def _checkTable(self):
    """_checkTable.

    Make sure the table is created
    """

    return self.__createTables()

  def __createTables(self):
    """__createTables.

    Writes the schema in the database. If a table is already in the
    schema, it is skipped to avoid problems trying to create a table
    that already exists.
    """

    # Horrible SQL here !!
    existingTables = self._query("show tables")
    if not existingTables['OK']:
      return existingTables
    existingTables = [existingTable[0] for existingTable in existingTables['Value']]

    # Makes a copy of the dictionary _tablesDict
    tables = {}
    tables.update(self._tablesDict)

    for existingTable in existingTables:
      if existingTable in tables:
        del tables[existingTable]

    res = self._createTables(tables)
    if not res['OK']:
      return res

    # Human readable S_OK message
    if res['Value'] == 0:
      res['Value'] = 'No tables created'
    else:
      res['Value'] = 'Tables created: %s' % (','.join(list(tables)))

      # Create the initial value
      if 'LastMonitor' in tables:
        ret = self._query("INSERT INTO LastMonitor(LastMonitorTime) value (UTC_TIMESTAMP());")
        if not ret['OK']:
          return ret

    return res

  def showTables(self):
    """return the list of tables."""

    existingTables = self._query("show tables")
    if not existingTables['OK']:
      return existingTables
    existingTables = [existingTable[0] for existingTable in existingTables['Value']]
    return S_OK(existingTables)

  def getActiveFiles(self):
    """Obtain all the active files in the database along with all their
    associated metadata."""
    try:
      gLogger.info(
          "RAWIntegrityDB.getActiveFiles: "
          "Obtaining files awaiting migration from database."
      )
      res = self._query(
          "SELECT "
          "LFN,PFN,Size,StorageElement,GUID,FileChecksum,TIME_TO_SEC(TIMEDIFF(UTC_TIMESTAMP(),SubmitTime)) "
          "from Files WHERE Status = 'Active';"
      )
      if not res['OK']:
        gLogger.error("RAWIntegrityDB.getActiveFiles: Failed to get files from database.",
                      res['Message'])
        return res
      else:
        fileDict = {}
        for lfn, pfn, size, se, guid, checksum, waittime in res['Value']:
          fileDict[lfn] = {
              'PFN': pfn,
              'Size': size,
              'SE': se,
              'GUID': guid,
              'Checksum': checksum,
              'WaitTime': waittime
          }
        gLogger.info(
            "RAWIntegrityDB.getActiveFiles: Obtained %s files awaiting migration from database." %
            len(fileDict))
        return S_OK(fileDict)
    except Exception as x:
      errStr = "RAWIntegrityDB.getActiveFiles: Exception while getting files from database."
      gLogger.exception(errStr, lException=x)
      return S_ERROR(errStr)

  def getFiles(self, status):
    """Obtain all the files in a give status (or list) in the database along
    with all their associated metadata.

    :param status: string or list of status we want to query

    :returns: dict {lfn: metadata}
    """
    if isinstance(status, six.string_types):
      status = [status]

    try:
      gLogger.info("RAWIntegrityDB.getActiveFiles: Obtaining files in status %s." % status)
      res = self._query(
          "SELECT "
          "LFN,Status,PFN,Size,StorageElement,GUID,FileChecksum,"
          "TIME_TO_SEC(TIMEDIFF(UTC_TIMESTAMP(),SubmitTime)),SubmitTime "
          "from Files WHERE Status in (%s);" % (stringListToString(status))
      )
      if not res['OK']:
        gLogger.error("RAWIntegrityDB.getActiveFiles: Failed to get files from database.",
                      res['Message'])
        return res
      fileDict = {}
      for lfn, st, pfn, size, se, guid, checksum, waittime, submitTime in res['Value']:
        fileDict[lfn] = {
            'PFN': pfn,
            'Size': size,
            'SE': se,
            'GUID': guid,
            'Checksum': checksum,
            'WaitTime': waittime,
            'Status': st,
            'SubmitTime': submitTime
        }
      gLogger.info(
          "RAWIntegrityDB.getActiveFiles: Obtained %s files awaiting migration from database." %
          len(fileDict))
      return S_OK(fileDict)
    except Exception as x:
      errStr = "RAWIntegrityDB.getActiveFiles: Exception while getting files from database."
      gLogger.exception(errStr, lException=x)
      return S_ERROR(errStr)

  def getUnmigratedFiles(self):
    """Get files in status Active, Copied, and Registered."""

    return self.getFiles(['Active', 'Copied', 'Registered'])

  def setFileStatus(self, lfn, status):
    """Update the status of the file in the database.

    A file in status 'Done' cannot be unchanged
    """
    try:
      gLogger.info("RAWIntegrityDB.setFileStatus: Attempting to update status of %s to '%s'." %
                   (lfn, status))
      req = "UPDATE Files SET Status='%s',CompleteTime=UTC_TIMESTAMP() WHERE LFN = '%s' AND Status != 'Done';" % (
          status, lfn)
      res = self._update(req)
      if not res['OK']:
        gLogger.error("RAWIntegrityDB.setFileStatus: Failed update file status.", res['Message'])
      else:
        gLogger.info("RAWIntegrityDB.setFileStatus: Successfully updated file status.")
      return res
    except Exception as x:
      errStr = "RAWIntegrityDB.setFileStatus: Exception while updating file status."
      gLogger.exception(errStr, lException=x)
      return S_ERROR(errStr)

  def removeFile(self, lfn):
    """Remove file from the DB."""
    try:
      gLogger.info("RAWIntegrityDB.removeFile: Attempting to remove %s." % lfn)
      req = "DELETE FROM Files WHERE LFN = '%s';" % lfn
      res = self._update(req)
      if not res['OK']:
        gLogger.error("RAWIntegrityDB.removeFile: Failed to remove file.", res['Message'])
      else:
        gLogger.info("RAWIntegrityDB.removeFile: Successfully removed file.")
      return res
    except Exception as x:
      errStr = "RAWIntegrityDB.removeFile: Exception while removing file."
      gLogger.exception(errStr, lException=x)
      return S_ERROR(errStr)

  def addFile(self, lfn, pfn, size, se, guid, checksum):
    """Insert file into the database."""
    try:
      gLogger.info("RAWIntegrityDB.addFile: Attempting to add %s to database." % lfn)
      req = "INSERT INTO Files (LFN,PFN,Size,StorageElement,GUID,FileChecksum,SubmitTime) VALUES\
            ('%s','%s',%s,'%s','%s','%s',UTC_TIMESTAMP());" % (lfn, pfn, size, se, guid, checksum)
      res = self._update(req)
      if not res['OK']:
        gLogger.error("RAWIntegrityDB.addFile: Failed update add file to database.", res['Message'])
      else:
        gLogger.info("RAWIntegrityDB.addFile: Successfully added file.")
      return res
    except Exception as x:
      errStr = "RAWIntegrityDB.addFile: Exception while updating file status."
      gLogger.exception(errStr, lException=x)
      return S_ERROR(errStr)

  def setLastMonitorTime(self):
    """Set the last time the migration rate was calculated."""
    try:
      gLogger.info(
          "RAWIntegrityDB.setLastMonitorTime: Attempting to set the last migration marker.")
      req = "UPDATE LastMonitor SET LastMonitorTime=UTC_TIMESTAMP();"
      res = self._update(req)
      if not res['OK']:
        gLogger.error("RAWIntegrityDB.setLastMonitorTime: Failed update migration marker.",
                      res['Message'])
      else:
        gLogger.info("RAWIntegrityDB.setLastMonitorTime: Successfully updated migration marker.")
      return res
    except Exception as x:
      errStr = "RAWIntegrityDB.setLastMonitorTime: Exception while updating migration marker."
      gLogger.exception(errStr, lException=x)
      return S_ERROR(errStr)

  def getLastMonitorTimeDiff(self):
    """Get the last time the migration rate was calculated."""
    try:
      gLogger.info(
          "RAWIntegrityDB.getLastMonitorTimeDiff: Attempting to get the last migration marker.")
      req = "SELECT TIME_TO_SEC(TIMEDIFF(UTC_TIMESTAMP(),LastMonitorTime)) FROM LastMonitor LIMIT 1;"
      res = self._query(req)
      if not res['OK']:
        gLogger.error("RAWIntegrityDB.getLastMonitorTimeDiff: Failed get migration marker.",
                      res['Message'])
        return res
      else:
        gLogger.info(
            "RAWIntegrityDB.getLastMonitorTimeDiff: Successfully obtained migration marker.")
        timediff = res['Value'][0][0]
        return S_OK(timediff)
    except Exception as x:
      errStr = "RAWIntegrityDB.getLastMonitorTimeDiff: Exception while getting migration marker."
      gLogger.exception(errStr, lException=x)
      return S_ERROR(errStr)

  def getGlobalStatistics(self):
    """Get the count of the file statutes in the DB."""
    req = "SELECT Status,COUNT(*) FROM Files GROUP BY Status;"
    res = self._query(req)
    if not res['OK']:
      return res
    statusDict = {}
    for resValue in res['Value']:
      status, count = resValue
      statusDict[status] = count
    return S_OK(statusDict)

  def getFileSelections(self):
    """Get the unique values of the selection fields."""
    selDict = {'StorageElement': [], 'Status': []}
    req = "SELECT DISTINCT(StorageElement) FROM Files;"
    res = self._query(req)
    if not res['OK']:
      return res
    for resValue in res['Value']:
      selDict['StorageElement'].append(resValue[0])
    req = "SELECT DISTINCT(Status) FROM Files;"
    res = self._query(req)
    if not res['OK']:
      return res
    for resValue in res['Value']:
      selDict['Status'].append(resValue[0])
    return S_OK(selDict)

  def __buildCondition(self, condDict, older=None, newer=None, timeStamp='SubmitTime'):
    """build SQL condition statement from provided condDict and other extra
    conditions."""
    condition = ''
    conjunction = "WHERE"
    if condDict is not None:
      for attrName, attrValue in condDict.items():
        ret = self._escapeString(attrName)
        if not ret['OK']:
          return ret
        attrName = "`" + ret['Value'][1:-1] + "`"
        if isinstance(attrValue, list):
          multiValueList = []
          for x in attrValue:
            ret = self._escapeString(x)
            if not ret['OK']:
              return ret
            x = ret['Value']
            multiValueList.append(x)
          multiValue = ','.join(multiValueList)
          condition = ' %s %s %s in (%s)' % (condition, conjunction, attrName, multiValue)
        else:
          ret = self._escapeString(attrValue)
          if not ret['OK']:
            return ret
          attrValue = ret['Value']
          condition = ' %s %s %s=%s' % (condition, conjunction, attrName, attrValue)
        conjunction = "AND"
    if older:
      ret = self._escapeString(older)
      if not ret['OK']:
        return ret
      older = ret['Value']
      condition = ' %s %s %s <= %s' % (condition, conjunction, timeStamp, older)
      conjunction = "AND"
    if newer:
      ret = self._escapeString(newer)
      if not ret['OK']:
        return ret
      newer = ret['Value']
      condition = ' %s %s %s >= %s' % (condition, conjunction, timeStamp, newer)
    return condition

  def selectFiles(self,
                  selectDict,
                  orderAttribute='LFN',
                  newer=None,
                  older=None,
                  limit=None):
    """Select the files which match the selection criteria."""
    condition = self.__buildCondition(selectDict, older, newer)
    if orderAttribute:
      orderType = None
      orderField = orderAttribute
      if orderAttribute.find(':') != -1:
        orderType = orderAttribute.split(':')[1].upper()
        orderField = orderAttribute.split(':')[0]
      condition = condition + ' ORDER BY ' + orderField
      if orderType:
        condition = condition + ' ' + orderType
    if limit:
      condition = condition + ' LIMIT ' + str(limit)
    res = self._query(
        "SELECT "
        "LFN,PFN,Size,StorageElement,GUID,FileChecksum,SubmitTime,CompleteTime,Status "
        "from Files %s" % condition
    )
    if not res['OK']:
      return res
    if not len(res['Value']):
      return S_OK([])
    return S_OK(res['Value'])


# ...............................................................................



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Create and send a combined request for any pending operations at the end of
a job:

fileReport (for the transformation) jobReport (for jobs) accounting
request (for failover)
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from DIRAC import S_OK, S_ERROR, gLogger

from LHCbDIRAC.Workflow.Modules.ModuleBase import ModuleBase

__RCSID__ = "$Id$"


class FailoverRequest(ModuleBase):

  #############################################################################

  def __init__(self, bkClient=None, dm=None):
    """Module initialization."""

    self.log = gLogger.getSubLogger("FailoverRequest")
    super(FailoverRequest, self).__init__(self.log, bkClientIn=bkClient, dm=dm)

    self.version = __RCSID__

  #############################################################################

  def _resolveInputVariables(self):
    """By convention the module input parameters are resolved here."""
    super(FailoverRequest, self)._resolveInputVariables()
    super(FailoverRequest, self)._resolveInputStep()

  #############################################################################

  def execute(self, production_id=None, prod_job_id=None, wms_job_id=None,
              workflowStatus=None, stepStatus=None,
              wf_commons=None, step_commons=None,
              step_number=None, step_id=None):
    """Main execution function."""

    try:

      super(FailoverRequest, self).execute(self.version, production_id, prod_job_id, wms_job_id,
                                           workflowStatus, stepStatus,
                                           wf_commons, step_commons, step_number, step_id)

      if not self._enableModule():
        return S_OK()

      self._resolveInputVariables()

      # preparing the request, just in case
      self.request.RequestName = 'job_%d_request.xml' % self.jobID
      self.request.JobID = self.jobID
      self.request.SourceComponent = "Job_%d" % self.jobID

      # report on the status of the input data, by default they are 'Processed', unless the job failed
      # failures happening before (e.g. in previous steps, or while inspecting the XML summary) are not touched.

      # The FileReport object is normally empty, unless there are some Problematic files,
      # or if there are files found to have descendants
      filesInFileReport = self.fileReport.getFiles()

      if not self._checkWFAndStepStatus(noPrint=True):
        # if the job fails, the input files will be reset to 'Unused'
        for lfn in self.inputDataList:
          if lfn not in filesInFileReport:
            self.log.info("Set status of input files to 'Unused' due to workflow failure", lfn)
            self.fileReport.setFileStatus(int(self.production_id), lfn, 'Unused')
      else:
        for lfn in self.inputDataList:
          if lfn not in filesInFileReport:
            self.log.verbose("No status populated for input data, setting to 'Processed'", lfn)
            self.fileReport.setFileStatus(int(self.production_id), lfn, 'Processed')

      res_fileReportCommit = self.fileReport.commit()
      # If there are still files to set, try a second time and generate a request if it fails again
      if not res_fileReportCommit['OK']:
        self.log.error("Something went wrong trying fileReport.commit()", res_fileReportCommit['Message'])
      else:
        if res_fileReportCommit['Value']:
          self.log.info("Status of files have been properly updated in the TransformationDB")
        else:
          self.log.warn("No file status update reported. There are no input files?")

      if self.fileReport.getFiles():
        self.log.error("On first attempt, failed to report file status to TransformationDB")
        # This will try a second time a commit, before generating a SetFileStatus operation
        result = self.fileReport.generateForwardDISET()
        if not result['OK']:
          self.log.warn(
              "Could not generate Operation for file report",
              "with result:\n%s" % (result['Message']))
        else:
          if result['Value'] is None:  # Means the FileReport managed to report, no need for a new operation
            self.log.info("On second attempt, files correctly reported to TransformationDB")
          else:
            self.log.error("On second attempt, SetFileStatus definitely failed")
            if self.workflowStatus['OK'] and self.stepStatus['OK']:  # the job will be in "Completing"
              self.log.info("Adding a SetFileStatus operation to the request")
              self.request.addOperation(result["Value"])
            else:
              self.log.info("The job should fail: do not set requests, as the DRA will take care")

      # Must ensure that the local job report instance is used to report the final status
      # in case of failure and a subsequent failover operation
      if self.workflowStatus['OK'] and self.stepStatus['OK']:
        self.setApplicationStatus("Job Finished Successfully")

      self.generateFailoverFile()

      return S_OK()

    except Exception as e:  # pylint:disable=broad-except
      self.log.exception("Failure in FailoverRequest execute module", lException=e)
      self.setApplicationStatus(repr(e))
      return S_ERROR(str(e))

    finally:
      super(FailoverRequest, self).finalize(self.version)

# EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""PluginScript module holds PluginScript class, which is an extension of
DMScript class."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from DIRAC import S_OK, gLogger
from DIRAC.Core.Base import Script
from DIRAC.DataManagementSystem.Utilities.DMSHelpers import resolveSEGroup

from LHCbDIRAC.DataManagementSystem.Client.DMScript import DMScript


class Setter(object):
  """Class used for setting an option: use setOption() method as a script
  setter."""

  def __init__(self, obj, name):
    self.name = name
    self.obj = obj

  def setOption(self, val):
    """Method used as a setter in the switch definition."""
    if self.name.endswith('='):
      try:
        self.obj.options[self.name[:-1]] = val if not self.name == "GroupSize=" else float(val)
      except ValueError as e:
        gLogger.exception("Bad value for parameter", self.name, lException=e)
    else:
      self.obj.options[self.name] = True

    return S_OK()


class PluginScript(DMScript):
  """Scripts utilities class."""

  def __init__(self):
    super(PluginScript, self).__init__()
    self.pluginParameters = {
        "Plugin=": "   Plugin name (mandatory)",
        "Type=": "   Transformation type [Replication] (Removal automatic)",
        "Parameters=": "   Additional plugin parameters ({<key>:<val>,[<key>:val>]}",
        "RequestID=": "   Sets the request ID (default 0)"
    }
    self.seParameters = ("KeepSEs", "ArchiveSEs",
                         "MandatorySEs", "SecondarySEs", "DestinationSEs", "FromSEs",
                         "RAWStorageElements", "ProcessingStorageElements",
                         )
    self.additionalParameters = {
        "GroupSize=": "   GroupSize parameter for merging (GB) or nb of files",
        "NumberOfReplicas=": "   Number of copies to create or to remove",
        "ProcessingPasses=": "   List of processing passes for the DeleteReplicasWhenProcessed plugin",
        "Period=": "   minimal period at which a plugin is executed (if instrumented)",
        "CleanTransformations": "   (only for DestroyDataset) clean transformations from the files being destroyed",
        'Debug': '   Sets a debug flag in the plugin',
        "UseRunDestination": "   for RAWReplication plugin, use the already defined run destination as storage"
    }
    self.setters = {}

  def registerPluginSwitches(self):
    """Set of switches used by TS plugins."""
    self.registerBKSwitches()

    for option in self.pluginParameters:
      self.setters[option] = Setter(self, option)
      Script.registerSwitch('', option, self.pluginParameters[option], self.setters[option].setOption)

    for param in self.seParameters:
      param += '='
      self.setters[param] = Setter(self, param)
      Script.registerSwitch("", param,
                            "   List of SEs for the corresponding parameter of the plugin",
                            self.setters[param].setOption)

    for option in self.additionalParameters:
      self.setters[option] = Setter(self, option)
      Script.registerSwitch('', option, self.additionalParameters[option], self.setters[option].setOption)

  def getPluginParameters(self):
    """Get  parameters used by TS plugins."""
    if 'Parameters' in self.options:
      params = eval(self.options['Parameters'])  # pylint: disable=eval-used
    else:
      params = {}
    # print self.options
    for key in set(self.options) & set(param if not param.endswith('=')
                                       else param[:-1]
                                       for param in self.additionalParameters):
      params[key] = self.options[key]
    return params

  def getPluginSEParameters(self):
    """Special treatment for SE-related parameters."""
    params = {}
    # print self.options
    for key in set(self.options) & set(self.seParameters):
      val = self.options[key]
      params[key] = val if val else []
    return params



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Unit tests for Workflow Module GaudiApplication."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import copy
import os

from mock import MagicMock
import pytest

from DIRAC.DataManagementSystem.Client.test.mock_DM import dm_mock
from LHCbDIRAC.BookkeepingSystem.Client.test.mock_BookkeepingClient import bkc_mock
from LHCbDIRAC.Workflow.Modules.mock_Commons import prod_id, prod_job_id, wms_job_id, \
    workflowStatus, stepStatus, step_id, step_number,\
    step_commons, wf_commons

# sut
from LHCbDIRAC.Workflow.Modules.GaudiApplication import GaudiApplication


@pytest.fixture
def rmFiles():
  yield
  try:
    os.remove('gaudi_extra_options.py')
  except OSError:
    pass


def test_execute(mocker, rmFiles):
  mocker.patch("LHCbDIRAC.Workflow.Modules.GaudiApplication.RunApplication", side_effect=MagicMock())
  mocker.patch("LHCbDIRAC.Workflow.Modules.GaudiApplication.ModuleBase._manageAppOutput", side_effect=MagicMock())
  mocker.patch("LHCbDIRAC.Workflow.Modules.GaudiApplication.gConfig", side_effect=MagicMock())
  mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.RequestValidator", side_effect=MagicMock())

  ga = GaudiApplication(bkClient=bkc_mock, dm=dm_mock)
  ga.siteName = 'LCG.PIPPO.org'
  ga.jobType = 'user'

  # no errors, no input data
  for wf_cs in copy.deepcopy(wf_commons):
    for s_cs in step_commons:
      assert ga.execute(prod_id, prod_job_id, wms_job_id,
                        workflowStatus, stepStatus,
                        wf_cs, s_cs,
                        step_number, step_id)['OK'] is True



#! /usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Get production numbers given a dataset path."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import time
import six
from DIRAC import gLogger, exit
from DIRAC.Core.Utilities.DIRACScript import DIRACScript


def printProds(title, prods):
  typeDict = {}
  for prod, prodType in prods.items():
    typeDict.setdefault(prodType, []).append(prod)
  gLogger.notice(title)
  for prodType, prodList in typeDict.items():
    gLogger.notice('(%s): %s' % (prodType, ','.join([str(prod) for prod in sorted(prodList)])))


def execute(dmScript):
  from DIRAC.Core.Base import Script
  from LHCbDIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

  tr = TransformationClient()

  for switch in Script.getUnprocessedSwitches():
    pass

  bkQuery = dmScript.getBKQuery()
  if not bkQuery:
    gLogger.notice("No BKQuery given...")
    exit(1)

  startTime = time.time()
  prods = bkQuery.getBKProductions()  # visible = 'All' )

  parents = {}
  productions = {}
  for prod in prods:
    ptype = tr.getTransformation(prod).get('Value', {}).get('Type', 'Unknown')
    productions[prod] = ptype
    parent = tr.getBookkeepingQuery(prod).get('Value', {}).get('ProductionID', '')
    while isinstance(parent, six.integer_types):
      ptype = tr.getTransformation(parent).get('Value', {}).get('Type', 'Unknown')
      parents[parent] = ptype
      parent = tr.getBookkeepingQuery(parent).get('Value', {}).get('ProductionID', '')

  gLogger.notice("For BK path %s:" % bkQuery.getPath())
  if not prods:
    gLogger.notice('No productions found!')
  else:
    printProds('Productions found', productions)
    if parents:
      printProds('Parent productions', parents)

  gLogger.notice('Completed in %.1f seconds' % (time.time() - startTime))


@DIRACScript()
def main():
  from DIRAC.Core.Base import Script
  from LHCbDIRAC.DataManagementSystem.Client.DMScript import DMScript

  dmScript = DMScript()
  dmScript.registerBKSwitches()
  Script.setUsageMessage(__doc__ + '\n'.join([
      'Usage:',
      '  %s [option|cfgfile]' % Script.scriptName, ]))

  Script.parseCommandLine(ignoreErrors=False)
  execute(dmScript)


if __name__ == "__main__":
  main()



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import sys
from time import time

from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from DIRAC.Core.Base import Script
  from DIRAC import exit, gLogger

  Script.registerSwitch('', 'User=', '  User name')
  Script.registerSwitch('', 'Recursive', '  Set ownership recursively')
  Script.registerSwitch('', 'Directory=', '   Directory to change the ownership of')
  Script.registerSwitch('', 'Group=', '   Group name (default lhcb_user)')
  Script.registerSwitch('', 'Mode=', '   Change permission mode (default: not changed)')
  Script.registerSwitch('', 'Create', '   Use for creating the directory if it does not exist')

  Script.parseCommandLine()

  user = None
  recursive = False
  group = None
  dirList = None
  mode = None
  create = False
  for switch in Script.getUnprocessedSwitches():
    if switch[0] == 'User':
      user = switch[1].lower()
    elif switch[0] == 'Recursive':
      recursive = True
    elif switch[0] == 'Directory':
      dirList = switch[1].split(',')
    elif switch[0] == 'Group':
      group = switch[1]
    elif switch[0] == 'Mode':
      mode = int(switch[1], base=8)
    elif switch[0] == 'Create':
      create = True

  args = Script.getPositionalArgs()
  if dirList is None and args:
    dirList = []
    for arg in args:
      dirList += arg.split(',')

  toChange = []
  if user:
    toChange.append('user')
  if group:
    toChange.append('group')
  if mode:
    toChange.append('mode')
  toChange = ','.join(toChange)

  from DIRAC.Core.Security.ProxyInfo import getProxyInfo
  from LHCbDIRAC.DataManagementSystem.Utilities.FCUtilities import chown

  res = getProxyInfo()
  if not res['OK']:
    gLogger.fatal("Can't get proxy info", res['Message'])
    exit(1)
  properties = res['Value'].get('groupProperties', [])
  if 'FileCatalogManagement' not in properties:
    gLogger.error("You need to use a proxy from a group with FileCatalogManagement")
    exit(5)

  if group and not group.startswith('lhcb_'):
    gLogger.fatal("This is not a valid group", group)
    exit(1)

  directories = set()
  for baseDir in dirList:
    if os.path.exists(baseDir):
      f = open(baseDir)
      directories.update([os.path.dirname(line.split()[0] + '/') for line in f.read().splitlines()])
      f.close()
    else:
      directories.add(baseDir if not baseDir.endswith('/') else baseDir[:-1])

  if not directories or (user is None and group is None and mode is None):
    Script.showHelp(exitCode=1)

  from LHCbDIRAC.DataManagementSystem.Client.DMScript import ProgressBar
  from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
  dfc = FileCatalogClient()

  success = 0
  startTime = time()
  progressBar = ProgressBar(len(directories),
                            title="Changing %s%s to %d directories:" % (toChange,
                                                                        ' recursively' if recursive else '',
                                                                        len(directories)),
                            chunk=1, interactive=True)
  errors = {}
  for baseDir in directories:
    progressBar.loop()
    if not baseDir.startswith('/lhcb'):
      errors[baseDir] = 'Not a valid directory'
      continue
    if not dfc.isDirectory(baseDir).get('Value', {}).get('Successful', {}).get(baseDir):
      if create:
        res = dfc.createDirectory(baseDir)
        if not res['OK']:
          errors[baseDir] = res['Message'] + ' while creating directory'
          continue
      else:
        errors[baseDir] = "Directory doesn't exist"
        continue
    res = chown(baseDir, user, group=group, mode=mode, recursive=recursive, fcClient=dfc)
    if not res['OK']:
      errors[baseDir] = res['Message'] + ' executing %s: ' % res['Action']
    else:
      success += res['Value']

  if success:
    msg = 'Successfully changed %s%s in %d directories' % (
        toChange, ' recursively' if recursive else '', success
    )
  else:
    msg = 'Failed changing %s%s in %d directories' % (
        toChange, ' recursively' if recursive else '', len(directories)
    )
  progressBar.endLoop(msg)
  retCode = 0
  if errors:
    retCode = 1
    gLogger.notice("Errors:")
    for baseDir, error in errors.items():
      gLogger.notice("\tDirectory %s - " % baseDir, error)
  elif len(directories) == 1:
    from LHCbDIRAC.DataManagementSystem.Client.DMScript import DMScript
    from LHCbDIRAC.DataManagementSystem.Client.ScriptExecutors import executeLfnMetadata
    dmScript = DMScript()
    dmScript.setLFNs(list(directories)[0])
    sys.stdout.write('Directory metadata: ')
    sys.stdout.flush()
    executeLfnMetadata(dmScript)
  exit(retCode)


if __name__ == "__main__":
  main()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Gaudi Application Script Class.

This allows the execution of a simple python script in a given LHCb project environment,
e.g. python <script> <arguments>. GaudiPython / Bender scripts can be executed very simply
in this way.

To make use of this module the LHCbJob method setApplicationScript can be called by users.

This is usually the main module run by user jobs.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import re
import os

from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.Utilities import DErrno

from LHCbDIRAC.Core.Utilities.RunApplication import RunApplication, LbRunError, LHCbApplicationError, LHCbDIRACError
from LHCbDIRAC.Workflow.Modules.ModuleBase import ModuleBase


class GaudiApplicationScript(ModuleBase):

  #############################################################################

  def __init__(self, bkClient=None, dm=None):
    self.version = __RCSID__
    self.log = gLogger.getSubLogger("GaudiApplicationScript")
    super(GaudiApplicationScript, self).__init__(self.log, bkClientIn=bkClient, dm=dm)

    # Set defaults for all workflow parameters here
    self.script = None
    self.arguments = ''
    self.systemConfig = ''
    self.applicationLog = ''
    self.applicationName = ''
    self.applicationVersion = ''
    self.poolXMLCatName = 'pool_xml_catalog.xml'

  #############################################################################

  def _resolveInputVariables(self):
    """By convention the workflow parameters are resolved here."""

    super(GaudiApplicationScript, self)._resolveInputVariables()
    super(GaudiApplicationScript, self)._resolveInputStep()

    if 'script' in self.step_commons:
      self.script = self.step_commons['script']
    else:
      self.log.warn('No script defined')

    if 'arguments' in self.step_commons:
      self.arguments = self.step_commons['arguments']

    if 'poolXMLCatName' in self.step_commons:
      self.poolXMLCatName = self.step_commons['poolXMLCatName']

  #############################################################################

  def execute(self, production_id=None, prod_job_id=None, wms_job_id=None,
              workflowStatus=None, stepStatus=None,
              wf_commons=None, step_commons=None,
              step_number=None, step_id=None):
    """The main execution method of the module.

    It runs a gaudi script app using RunApplication module. This is
    usually the main module run by user jobs.
    """

    try:

      super(GaudiApplicationScript, self).execute(self.version,
                                                  production_id, prod_job_id, wms_job_id,
                                                  workflowStatus, stepStatus,
                                                  wf_commons, step_commons,
                                                  step_number, step_id)

      self._resolveInputVariables()

      if not self.applicationName or not self.applicationVersion:
        raise RuntimeError('No Gaudi Application defined')
      if not self.systemConfig:
        raise RuntimeError('No binary tag selected')
      if not self.script:
        raise RuntimeError('No script defined')
      if not self.applicationLog:
        self.applicationLog = '%s.log' % (os.path.basename(self.script))

      self.log.info("Executing application %s %s for binary tag %s" % (self.applicationName,
                                                                       self.applicationVersion,
                                                                       self.systemConfig))

      gaudiCmd = []
      if re.search('.py$', self.script):
        gaudiCmd.append('python')
        gaudiCmd.append(os.path.basename(self.script))
        gaudiCmd.append(self.arguments)
      else:
        gaudiCmd.append(os.path.basename(self.script))
        gaudiCmd.append(self.arguments)
      command = ' '.join(gaudiCmd)

      # How to run the application
      ra = RunApplication()
      # lb-run stuff
      ra.applicationName = self.applicationName
      ra.applicationVersion = self.applicationVersion
      ra.systemConfig = self.systemConfig
      # actual stuff to run
      ra.command = command
      ra.applicationLog = self.applicationLog
      ra.numberOfProcessors = self.numberOfProcessors

      # Now really running
      self.setApplicationStatus(self.applicationName)
      ra.run()  # This would trigger an exception in case of failure, or application status != 0

      self.setApplicationStatus('%s Successful' % os.path.basename(self.script))
      return S_OK('%s Successful' % os.path.basename(self.script))

    except LbRunError as lbre:  # This is the case for lb-run/environment errors
      self.setApplicationStatus(repr(lbre))
      return S_ERROR(DErrno.EWMSRESC, str(lbre))
    except LHCbApplicationError as lbae:  # This is the case for real application errors
      self.setApplicationStatus(repr(lbae))
      return S_ERROR(str(lbae))
    except LHCbDIRACError as lbde:  # This is the case for LHCbDIRAC errors (e.g. subProcess call failed)
      self.setApplicationStatus(repr(lbde))
      return S_ERROR(str(lbde))
    except Exception as e:  # pylint:disable=broad-except
      self.log.exception("Failure in GaudiApplicationScript execute module", lException=e)
      self.setApplicationStatus("Error in GaudiApplicationScript module")
      return S_ERROR(str(e))
    finally:
      super(GaudiApplicationScript, self).finalize(self.version)



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
########################################################################
# File :    dirac-wms-job-parameters
# Author :  Philippe Charpentier
########################################################################
"""
  Retrieve parameters associated to the given DIRAC job
"""

from __future__ import print_function

__RCSID__ = "$Id$"

import DIRAC

from DIRAC import gLogger, S_OK
from DIRAC.Core.Base import Script

from LHCbDIRAC.DataManagementSystem.Client.DMScript import printDMResult

Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s [option|cfgfile] ... JobID ...' % Script.scriptName,
                                  'Arguments:',
                                  '  JobID:    DIRAC Job ID']))
Script.parseCommandLine(ignoreErrors=True)
args = Script.getPositionalArgs()

if len(args) < 1:
  Script.showHelp(exitCode=1)

from DIRAC.Interfaces.API.Dirac import Dirac, parseArguments
dirac = Dirac()
exitCode = 0

results = {'Successful': {}, 'Failed': {}}
success = results['Successful']
failed = results['Failed']
for job in parseArguments(args):

  result = dirac.getJobParameters(job, printOutput=False)
  if not result['OK']:
    failed[job] = result['Message']
    exitCode = 2
  else:
    success[job] = result['Value']

printDMResult(S_OK(results))

DIRAC.exit(exitCode)



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
""" Module containing a front-end to the ElasticSearch-based ElasticMCGaussLogErrorsDB.
    This module interacts with one ES index: "ElasticMCLogErrors",

    Here we define a mapping which is taken from a list of log errors.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


__RCSID__ = "$Id$"

from DIRAC import S_OK, S_ERROR
from DIRAC.Core.Base.ElasticDB import ElasticDB


class ElasticMCStatsDBBase(ElasticDB):

  def set(self, data):
    """
    Inserts data into ElasticJobParametersDB index

    :param self: self reference
    :param str value: data to be inserted

    :returns: S_OK/S_ERROR as result of indexing
    """

    self.log.debug(self.__class__.__name__,
                   '.set(): inserting data in %s:%s' % (self.indexName, data))  # pylint: disable=no-member

    result = self.index(self.indexName,  # pylint: disable=no-member
                        body=data,
                        docID=str(data['ProductionID']) + '_' + str(data['JobID']))
    if not result['OK']:
      self.log.error("ERROR: Couldn't insert data", result['Message'])
    return result

  def get(self, productionID):
    """ Get docs per productionID. Basically here only for tests, right now

    :param self: self reference
    :param int productionID: production ID

    :return: dict with all docs
    """

    self.log.debug(self.__class__.__name__ + '.get(): Getting for production %s' % str(productionID))

    resultList = []

    """ the following should be equivalent to
    {
      "query": {
        "bool": {
          "filter": {  # no scoring
            "term": {"ProductionID": productionID}  # term level query, does not pass through the analyzer
          }
        }
      }
    }
    """

    s = self.dslSearch.query("bool", filter=self._Q("term", ProductionID=productionID))  # pylint: disable=no-member

    res = s.execute()

    for hit in res:
      hitDict = {}
      for name in hit:
        hitDict[name] = getattr(hit, name)
      resultList.append(hitDict)

    return S_OK(resultList)

  def remove(self, productionID):
    """ Remove docs per productionID. Basically here only for tests, right now

    :param self: self reference
    :param int productionID: production ID

    :return: S_OK/S_ERROR
    """

    self.log.debug(self.__class__.__name__ + '.remove(): Removing documents of production %s' % str(productionID))

    """ the following should be equivalent to
    {
      "query": {
        "bool": {
          "filter": {  # no scoring
            "term": {"ProductionID": productionID}  # term level query, does not pass through the analyzer
          }
        }
      }
    }
    """

    s = self.dslSearch.query("bool", filter=self._Q("term", ProductionID=productionID))  # pylint: disable=no-member
    try:
      s.delete()
    except Exception as e:
      self.log.exception()
      return S_ERROR(repr(e))

    return S_OK()
